unit DCGHighlighter;

// this unit contains the implementation of a lexical analyzer
// see documentation for an extensive description

interface

uses Classes, Graphics, Highlighter;

const CR = #13;
      LF = #10;

// user defined start states and tokens



const kwFirstKeyword = 257;
      kwABSOLUTE = 257;
      kwABSTRACT = 258;
      kwAND = 259;
      kwARRAY = 260;
      kwAS = 261;
      kwASM = 262;
      kwASSEMBLER = 263;
      kwAT = 264;
      kwAUTOMATED = 265;
      kwBEGIN = 266;
      kwCASE = 267;
      kwCDECL = 268;
      kwCLASS = 269;
      kwCONST = 270;
      kwCONSTRUCTOR = 271;
      kwCONTAINS = 272;
      kwDEFAULT = 273;
      kwDESTRUCTOR = 274;
      kwDISPID = 275;
      kwDISPINTERFACE = 276;
      kwDIV = 277;
      kwDO = 278;
      kwDOWNTO = 279;
      kwDYNAMIC = 280;
      kwELSE = 281;
      kwEND = 282;
      kwEXCEPT = 283;
      kwEXPORT = 284;
      kwEXPORTS = 285;
      kwEXTERNAL = 286;
      kwFAR = 287;
      kwFILE = 288;
      kwFINALIZATION = 289;
      kwFINALLY = 290;
      kwFOR = 291;
      kwFORWARD = 292;
      kwFUNCTION = 293;
      kwGOTO = 294;
      kwIF = 295;
      kwIMPLEMENTATION = 296;
      kwIMPLEMENTS = 297;
      kwIN = 298;
      kwINDEX = 299;
      kwINHERITED = 300;
      kwINITIALIZATION = 301;
      kwINLINE = 302;
      kwINTERFACE = 303;
      kwIS = 304;
      kwLABEL = 305;
      kwLIBRARY = 306;
      kwMESSAGE = 307;
      kwMOD = 308;
      kwNAME = 309;
      kwNEAR = 310;
      kwNIL = 311;
      kwNODEFAULT = 312;
      kwNOT = 313;
      kwOBJECT = 314;
      kwOF = 315;
      kwON = 316;
      kwOR = 317;
      kwOUT = 318;
      kwOVERLOAD = 319;
      kwOVERRIDE = 320;
      kwPACKAGE = 321;
      kwPACKED = 322;
      kwPASCAL = 323;
      kwPRIVATE = 324;
      kwPROCEDURE = 325;
      kwPROGRAM = 326;
      kwPROPERTY = 327;
      kwPROTECTED = 328;
      kwPUBLIC = 329;
      kwPUBLISHED = 330;
      kwRAISE = 331;
      kwREAD = 332;
      kwREADONLY = 333;
      kwRECORD = 334;
      kwREGISTER = 335;
      kwREINTRODUCE = 336;
      kwREPEAT = 337;
      kwREQUIRES = 338;
      kwRESIDENT = 339;
      kwRESOURCESTRING = 340;
      kwSAFECALL = 341;
      kwSET = 342;
      kwSHL = 343;
      kwSHR = 344;
      kwSTDCALL = 345;
      kwSTORED = 346;
      kwSTRING = 347;
      kwTHEN = 348;
      kwTHREADVAR = 349;
      kwTO = 350;
      kwTRY = 351;
      kwTYPE = 352;
      kwUNIT = 353;
      kwUNTIL = 354;
      kwUSES = 355;
      kwVAR = 356;
      kwVIRTUAL = 357;
      kwWHILE = 358;
      kwWITH = 359;
      kwWRITE = 360;
      kwWRITEONLY = 361;
      kwLastKeyword = 361;

const ASMTOKEN = 362;
      DCGCOMMENT = 363;
      DCGKEYWORD = 364;
      DCGMACRO = 365;
      IDENTIFIER = 366;
      MLCOMMENT = 367;
      NUMBER = 368;
      SLCOMMENT = 369;
      STRINGCONSTANT = 370;
      SYMBOL = 371;
      UNKNOWN = 372;
      WHITESPACE = 373;



type
  TDCGHighlighter = class(TBaseHighlighter)
  private
    FCurrentToken: Integer;            // type of current token
    FInput: PChar;                     // input string to be tokenized
    FCurrentPos: Cardinal;             // current lexer position in FInput
    FTokenPos: Cardinal;               // current token position in FInput
    FLookahead: Char;                  // next char after current position
    FCurrentState,
    FStartState,
    FLineState: Cardinal;              // lexer states
    FLastChar: Char;                   // last matched char (#0 if no match)
    FPositions: array of Cardinal;     // for each rule the last marked position, zeroed when
                                       // rule has already been considered (allocated once)
    FMatchStack: array of Cardinal;    // stack containing matched rules (growing dynamically)
    FMatchCount: Integer;              // contains current number of matches
    FRejected: Boolean;                // current match rejected?
    FDoStop: Boolean;                  // token or eof found?
    FEOL: Boolean;

    // !! adjust attributes !!
    FDCGKeywordAttributes,
    FDCGMacroAttributes,
    FDCGCommentAttributes,
    FAsmAttributes,
    FIdentifierAttributes,
    FNumberAttributes,
    FSpaceAttributes,
    FStringAttributes,
    FCommentAttributes,
    FSymbolAttributes,
    FKeyAttributes,
    FUnknownAttributes: THighlightAttributes;

    procedure RuleToToken(Rule: Integer);
    function GetNextChar: Char;
    function MatchRule(var Rule: Integer): Boolean;
    function DoDefault: Boolean;
    function GetCurrentChar: Char;
    procedure SetAttribute(const Index: Integer; const Value: THighlightAttributes);
    procedure HighlightChange(Sender: TObject);
  protected
    procedure Reset; virtual;
    procedure SetMatch(I: Integer);
    procedure SetState(Value: Cardinal); virtual;
    procedure SetToken(Token: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function EOL: Boolean; override;
    function GetToken: String; override;
    function GetTokenInfo: TTokenData; override;
    procedure Next; override;
    procedure Reject;
    function GetAttribute(Index: Integer): THighlightAttributes; override;
    function GetAttributeCount: Integer; override;
    function GetIdentChars: TIdentChars; override;
    function GetLanguageName: String; override;
    function GetRange: Pointer; override;
    function IsKeyword: Boolean;
    procedure ResetRange; override;
    procedure SetRange(Value: Pointer); override;
    procedure SetLine(const NewValue: String); override;

    property CurrentChar: Char read GetCurrentChar;
    property NextChar: Char read GetNextChar;
    property Lookahead: Char read FLookahead;
    property State: Cardinal read FCurrentState write SetState; // read to get current lexer state
                                                                // write to set start state for next run
    property TokenKind: Integer read FCurrentToken;
    property TokenPosition: Cardinal read FTokenPos;
  published
    property DCGKeywordAttributes: THighlightAttributes index 0 read FDCGKeywordAttributes write SetAttribute;
    property DCGMacroAttributes: THighlightAttributes index 0 read FDCGMacroAttributes write SetAttribute;
    property DCGCommentAttributes: THighlightAttributes index 0 read FDCGCommentAttributes write SetAttribute;
    property AsmAttributes: THighlightAttributes index 0 read FAsmAttributes write SetAttribute;
    property IdentifierAttributes: THighlightAttributes index 1 read FIdentifierAttributes write SetAttribute;
    property NumberAttributes: THighlightAttributes index 2 read FNumberAttributes write SetAttribute;
    property SpaceAttributes: THighlightAttributes index 3 read FSpaceAttributes write SetAttribute;
    property StringAttributes: THighlightAttributes index 4 read FStringAttributes write SetAttribute;
    property CommentAttributes: THighlightAttributes index 5 read FCommentAttributes write SetAttribute;
    property SymbolAttributes: THighlightAttributes index 6 read FSymbolAttributes write SetAttribute;
    property KeyAttributes: THighlightAttributes index 7 read FKeyAttributes write SetAttribute;
    property UnknownAttributes: THighlightAttributes index 8 read FUnknownAttributes write SetAttribute;
  end;

//------------------------------------------------------------------------------

implementation

uses SysUtils;

// DFA table


//------------------------------------------------------------------------------

constructor TDCGHighlighter.Create(AOwner: TComponent);

begin
  inherited;
  // allocate positions array (length is highest possible mark position)
  SetLength(FPositions, MarksHigh[StateCount - 1] + 1);

  DefaultFilter := 'Lexical description (*.l)|*.l|Parser description (*.y)|*.y|DCG code template (*.cod)|*.cod|';

  // !! adjust attributes !!
  FDCGKeywordAttributes := THighlightAttributes.Create('DCG Keyword');
  FDCGKeywordAttributes.OnChange := HighlightChange;
  FDCGMacroAttributes := THighlightAttributes.Create('DCG Macro');
  FDCGMacroAttributes.OnChange := HighlightChange;
  FDCGCommentAttributes := THighlightAttributes.Create('DCG Comment');
  FDCGCommentAttributes.OnChange := HighlightChange;

  FAsmAttributes := THighlightAttributes.Create('Assembler');
  FAsmAttributes.OnChange := HighlightChange;
  FIdentifierAttributes := THighlightAttributes.Create('Identifier');
  FIdentifierAttributes.OnChange := HighlightChange;
  FNumberAttributes := THighlightAttributes.Create('Number');
  FNumberAttributes.OnChange := HighlightChange;
  FSpaceAttributes := THighlightAttributes.Create('White space');
  FSpaceAttributes.OnChange := HighlightChange;
  FStringAttributes := THighlightAttributes.Create('String');
  FStringAttributes.OnChange := HighlightChange;
  FCommentAttributes := THighlightAttributes.Create('Comment');
  FCommentAttributes.OnChange := HighlightChange;
  FSymbolAttributes := THighlightAttributes.Create('Symbol');
  FSymbolAttributes.OnChange := HighlightChange;
  FKeyAttributes := THighlightAttributes.Create('Keyword');
  FKeyAttributes.OnChange := HighlightChange;
  FUnknownAttributes := THighlightAttributes.Create('Unknown');
  FUnknownAttributes.OnChange := HighlightChange;
end;

//------------------------------------------------------------------------------

destructor TDCGHighlighter.Destroy;

begin
  FPositions := nil;
  FMatchStack := nil;

  // !! adjust attributes !!
  FDCGKeywordAttributes.Free;
  FDCGMacroAttributes.Free;
  FDCGCommentAttributes.Free;

  FAsmAttributes.Free;
  FIdentifierAttributes.Free;
  FNumberAttributes.Free;
  FSpaceAttributes.Free;
  FStringAttributes.Free;
  FCommentAttributes.Free;
  FSymbolAttributes.Free;
  FKeyAttributes.Free;
  FUnknownAttributes.Free;

  inherited;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.IsKeyword: Boolean;

// checks whether the current token is a keyword,
// if so then the current token type is set accordingly

var Word: String;

begin
  SetString(Word, FInput + FTokenPos, FCurrentPos - FTokenPos);
  FCurrentToken := GetKeywordToken(Word);
  if FCurrentToken > 0 then
  begin
    Result := True;
    FDoStop := True;
  end
  else Result := False;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.Eol: Boolean;

begin
  Result := (FCurrentToken = -1) or FEOL;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetToken: String;

var Len: Cardinal;

begin
  Len := FCurrentPos - FTokenPos;
  SetString(Result, FInput + FTokenPos, Len);
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.Reset;

begin
  FTokenPos := 0;
  FCurrentPos := 0;
  FCurrentToken := 0;
  FLineState := 1;
  FLastChar := #0;
  FMatchCount := 0;
  FEOL := False;
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.SetToken(Token: Integer);

begin
  FCurrentToken := Token;
  FDoStop := True;
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.RuleToToken(Rule: Integer);

// var S : String;

begin
  // SetString(S, FInput + FTokenPos, FCurrentPos - FTokenPos);
  // actions
  case Rule of
        SetToken(STRINGCONSTANT);

        SetToken(NUMBER);

        repeat
          case CurrentChar of
            '"',
            CR, #0:
              begin
                if CurrentChar = '"' then NextChar;
                SetToken(STRINGCONSTANT);
                Break;
              end;
            end;
            NextChar;
          until False;

        repeat
          case CurrentChar of
            '''',
            CR, #0:
              begin
                if CurrentChar = '''' then NextChar;
                SetToken(STRINGCONSTANT);
                Break;
              end;
            end;
          NextChar;
        until False;

        begin
          while not (CurrentChar in [CR, #0]) do NextChar;
          SetToken(SLCOMMENT);
        end;

        repeat
          case CurrentChar of
            '*':
              if Lookahead = ')' then
              begin
                SetToken(MLCOMMENT);
                // skip star and closing parenthesis
                NextChar;
                NextChar;
                Break;
              end;
            CR, #0:
              begin
                State := Comment3;
                SetToken(MLCOMMENT);
                Break;
              end;
          end;
          NextChar;
        until False;

        SetToken(MLCOMMENT);

        begin
          if CurrentChar = ')' then
          begin
            NextChar;
            State := Normal;
          end;
          SetToken(MLCOMMENT);
        end;

        repeat
          case CurrentChar of
            '}',
            CR, #0:
              begin
                if CurrentChar = '}' then NextChar
                                     else State := Comment;
                SetToken(MLCOMMENT);
              Break;
            end;
          end;
          NextChar;
        until False;

        SetToken(MLCOMMENT);

        begin
          SetToken(MLComment);
          State := Normal;
        end;

        SetToken(DCGKEYWORD);

        SetToken(DCGMACRO);

        SetToken(DCGCOMMENT);

        if CompareText(GetToken, 'asm') = 0 then
        begin
          State := AssemblerState;
          IsKeyword;
        end
        else
          if not IsKeyword then SetToken(IDENTIFIER);

        if CompareText(GetToken, 'end') = 0 then
        begin
          IsKeyword;
          State := Normal;
        end
        else SetToken(ASMTOKEN);

        repeat
          case CurrentChar of
            '}',
            CR, #0:
              begin
                if CurrentChar = '}' then NextChar
                                     else State := Comment2;
                SetToken(MLCOMMENT);
                Break;
              end;
          end;
          NextChar;
        until False;

        SetToken(MLCOMMENT);

        begin
          SetToken(MLComment);
          State := AssemblerState;
        end;

        SetToken(ASMTOKEN);

        SetToken(WHITESPACE);

        SetToken(SYMBOL);

        SetToken(UNKNOWN);

        SetToken(-1); // special token to mark input end

  end;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.MatchRule(var Rule : Integer ) : Boolean;

// finds the last match and the corresponding marked position and adjusts
// the matched string accordingly;
// returns:
// - True if a rule has been matched, False otherwise
// - Rule: the number of the matched rule

begin
  FRejected := False;
  while (FMatchCount > 0) and (FPositions[FMatchStack[FMatchCount - 1]] = 0) do Dec(FMatchCount);

  if FMatchCount > 0 then
  begin
    Rule := FMatchStack[FMatchCount - 1];
    FCurrentPos := FTokenPos + FPositions[Rule];
    FPositions[Rule] := 0;
    if (FCurrentPos - FTokenPos) > 0 then FLastChar := FInput[FCurrentPos]
                                     else FLastChar := #0;
    Result := True;
  end
  else
  begin
    // return position to starting point
    FCurrentPos := FTokenPos;
    FLastChar := #0;
    Result := False;
  end
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.DoDefault : Boolean;

begin
  FRejected := False;
  if NextChar <> #0 then Result := True
                    else
  begin
    FLineState := 1;
    Result := False;
  end;
  if not FEOL then FLastChar := FInput[FCurrentPos];
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.Next;

// scans for the next token and determines its kind (FCurrentToken and FTokenPos
// are set appropriately)

var I : Integer;
    Len : Cardinal;
    Matched : Boolean;
    TransLow,
    TransHigh: Integer;

begin
  FDoStop := False;

  repeat
    // initialize run
    FTokenPos := FCurrentPos;

    if FLastChar <> #0 then
      if FLastChar = LF then FLineState := 1
                        else FLineState := 0;
    FCurrentState := FStartState + FLineState;

    repeat // character scan loop
      // mark positions and matches
      // for I := MarksLow[FCurrentState] to MarksHigh[FCurrentState] do FPositions[MarkPositionTable[I]] := Len;
      // significant speed improvement by using assembler here:
      Len := FCurrentPos - FTokenPos;
      asm
         PUSH EDI
         PUSH ECX
         MOV EAX, Self
         MOV EDI, [EAX + FPositions]   // address of FPositon array
         MOV ECX, [EDI - 4]            // length of dynamic array (cannot call @DynArrayLength here)
         MOV EAX, [Len]
         REP STOSD
         POP ECX
         POP EDI
      end;
      for I := MatchesHigh[FCurrentState] downto MatchesLow[FCurrentState] do SetMatch(MatchTable[I]);

      TransLow := TransitionsLow[FCurrentState];
      TransHigh := TransitionsHigh[FCurrentState];

      // determine action
      while (TransLow <= TransHigh) and not (FInput[FCurrentPos] in TransitionTable[TransLow].CharClass) do Inc(TransLow);

      if TransLow > TransHigh then Break; // no transition on current char in this state

      // get next character
      NextChar;

      // switch to new state
      FCurrentState := TransitionTable[TransLow].NextState;

    until False;

    repeat
      Matched := MatchRule(I);
      if Matched then RuleToToken(I)
                 else Break;
      // FRejected can be set in RuleToToken by calling Reject
    until Matched and not FRejected;

    if not (Matched or DoDefault) then
    begin
      FStartState := Normal;
      FLineState := 1;
      FLastChar := #0;
      SetToken(0);
    end;
  until FDoStop;
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.SetLine(const NewValue: String);

begin
  FInput := PChar(NewValue);
  Reset;
  Next;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetNextChar: Char;

begin
  if not FEOL and (FInput[FCurrentPos] = #0) then FEOL := True;
  Inc(FCurrentPos);
  if not FEOL then
  begin
    FLookahead := FInput[FCurrentPos + 1];
    Result := FInput[FCurrentPos];
  end
  else Result := #0;
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.SetState(Value: Cardinal);

begin
  FStartState := Value;
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.SetMatch(I: Integer);

begin
  if FMatchCount >= Length(FMatchStack) then SetLength(FMatchStack, FMatchCount + 1);
  FMatchStack[FMatchCount] := I;
  Inc(FMatchCount);
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.Reject;

begin
  FRejected := True;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetTokenInfo: TTokenData;

// returns all relevant data regarding the current token (length, font styles etc.)

begin
  with Result do
  begin
    if FEOL then
    begin
      Token := StrEnd(FInput);
      Length := Token - FInput - Integer(FTokenPos);
    end
    else
    begin
      Token := FInput + FTokenPos;
      Length := FCurrentPos - FTokenPos;
    end;
    TokenType := FCurrentToken;
    Position := FTokenPos;

    case FCurrentToken of
      DCGKEYWORD:
        begin
          Background := FDCGKeywordAttributes.Background;
          Foreground := FDCGKeywordAttributes.Foreground;
          Style := FDCGKeywordAttributes.Style;
        end;
      DCGMACRO:
        begin
          Background := FDCGMacroAttributes.Background;
          Foreground := FDCGMacroAttributes.Foreground;
          Style := FDCGMacroAttributes.Style;
        end;
      DCGCOMMENT:
        begin
          Background := FDCGCommentAttributes.Background;
          Foreground := FDCGCommentAttributes.Foreground;
          Style := FDCGCommentAttributes.Style;
        end;
      ASMTOKEN:
        begin
          Background := FAsmAttributes.Background;
          Foreground := FAsmAttributes.Foreground;
          Style := FAsmAttributes.Style;
        end;
      IDENTIFIER:
        begin
          Background := FIdentifierAttributes.Background;
          Foreground := FIdentifierAttributes.Foreground;
          Style := FIdentifierAttributes.Style;
        end;
      NUMBER:
        begin
          Background := FNumberAttributes.Background;
          Foreground := FNumberAttributes.Foreground;
          Style := FNumberAttributes.Style;
        end;
      WHITESPACE:
        begin
          Background := FSpaceAttributes.Background;
          Foreground := FSpaceAttributes.Foreground;
          Style := FSpaceAttributes.Style;
        end;
      STRINGCONSTANT:
        begin
          Background := FStringAttributes.Background;
          Foreground := FStringAttributes.Foreground;
          Style := FStringAttributes.Style;
        end;
        SLCOMMENT,
        MLCOMMENT:
        begin
          Background := FCommentAttributes.Background;
          Foreground := FCommentAttributes.Foreground;
          Style := FCommentAttributes.Style;
        end;
      SYMBOL:
        begin
          Background := FSymbolAttributes.Background;
          Foreground := FSymbolAttributes.Foreground;
          Style := FSymbolAttributes.Style;
        end;
      kwFirstKeyword..kwLastKeyword:
        begin
          Background := FKeyAttributes.Background;
          Foreground := FKeyAttributes.Foreground;
          Style := FKeyAttributes.Style;
        end;
    else // UNKNOWN
        begin
          Background := FUnknownAttributes.Background;
          Foreground := FUnknownAttributes.Foreground;
          Style := FUnknownAttributes.Style;
        end;
    end;
  end;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetCurrentChar: Char;

begin
  Result := FInput[FCurrentPos];
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.HighlightChange(Sender:TObject);

begin
  WindowList.Invalidate;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetIdentChars: TIdentChars;

begin
  Result := ['_', '0'..'9', 'a'..'z', 'A'..'Z'];
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetAttributeCount: Integer;

begin
  Result := 12;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetAttribute(Index: Integer): THighlightAttributes;

begin
  // !! adjust attributes !!
  case Index of
    0:
      Result := FDCGKeywordAttributes;
    1:
      Result := FDCGMacroAttributes;
    2:
      Result := FDCGCommentAttributes;
    3:
      Result := FAsmAttributes;
    4:
      Result := FIdentifierAttributes;
    5:
      Result := FNumberAttributes;
    6:
      Result := FSpaceAttributes;
    7:
      Result := FStringAttributes;
    8:
      Result := FCommentAttributes;
    9:
      Result := FSymbolAttributes;
    10:
      Result := FKeyAttributes;
    11:
      Result := FUnknownAttributes;
    else Result := nil;
  end;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetLanguageName: String;

begin
  Result := 'DCG';
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.SetAttribute(const Index: Integer; const Value: THighlightAttributes);

begin
  // !! adjust attributes !!
  case Index of
    0:
      FDCGKeywordAttributes.Assign(Value);
    1:
      FDCGMacroAttributes.Assign(Value);
    2:
      FDCGCommentAttributes.Assign(Value);
    3:
      FAsmAttributes.Assign(Value);
    4:
      FIdentifierAttributes.Assign(Value);
    5:
      FNumberAttributes.Assign(Value);
    6:
      FSpaceAttributes.Assign(Value);
    7:
      FStringAttributes.Assign(Value);
    8:
      FCommentAttributes.Assign(Value);
    9:
      FSymbolAttributes.Assign(Value);
    10:
      FKeyAttributes.Assign(Value);
    11:
      FUnknownAttributes.Assign(Value);
  end;
end;

//------------------------------------------------------------------------------

function TDCGHighlighter.GetRange: Pointer;

begin
  Result := Pointer(FStartState);
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.SetRange(Value: Pointer);

begin
  FStartState := Integer(Value);
end;

//------------------------------------------------------------------------------

procedure TDCGHighlighter.ResetRange;

begin
  FStartState := Normal;
  // in case someone will ask for it immediately afterwards
  FCurrentState := Normal;
end;

//------------------------------------------------------------------------------

end.


