 Description file for the Delphi Compiler Generator (DCG)
 this file contains definitions and rules for a lexical analyzer
 to tokenize HTM source code

 Note: All identifiers which are lexer variables and/or definitions are case sensitive.
       Identifiers preceded by a "%" define lexer variables, all other define macros
       (except keywords of course).

 ---------- first section (global definitions) ----------
LexerName HTMLHighlighter     define name for the target module (unit name)
                              as well as for the lexer class itself (see also *.cod file)
Filter 'HTML files (*.htm, *.html)|*.htm;*.html'
Language 'HTML'
%CaseSensitive  FALSE         tell DCG whether to use case sensitive keywords (allowed are
                              T and TRUE all other values are considered as false,
                              "CaseSensitive" might be abbrevated with "CS")

%%
 ---------- keyword section ----------
 all following identifiers (until section end) are considered as keywords
 this list will automatically be sorted
 EMPTY SECTION

%%
 ---------- definition section ----------
 token definitions
%T IDENTIFIER INTEGERNUMBER FLOATNUMBER WHITESPACE STRINGCONSTANT
%T MLCOMMENT SYMBOL KEYWORD UNKNOWN UNKNOWNPSEUDO UNKNOWNEND
%T UNKNOWNPSEUDOEND UNKNOWNPARM URLSTRING DOCTYPE NAMETOKEN SCRIPTPART

 start state definitions
%S Normal
%S UnknownAttrs
%S AttrParm
%S Comment
%S CommentAlt
%S Script

unicode (\\[0-9a-fA-F]{1,4})
latin1  ([\200-\245])                       values are octal based!
escape  (unicode|\\[ -~\200-\245])
nmstrt  ([a-zA-Z_]|latin1|escape)
nmchar  ([\-a-zA-Z0-9_]|latin1|escape)
ident   (nmstrtnmchar*)
eof     \0

%%
 ---------- rules section ----------

<Normal>[^<&]+                        SetToken(IDENTIFIER);
<Normal>&#[0-9]+;?                    SetToken(INTEGERNUMBER);   include tolerance for trailing ";"
<Normal>&[A-Za-z]+;?                  SetToken(STRINGCONSTANT);  include tolerance for trailing ";"
<Normal><[Ss][Cc][Rr][Ii][Pp][Tt]     begin
                                        State := Script;
                                        SetToken(SCRIPTPART);
                                      end;
<Normal><ident>                     SetToken(KEYWORD); // SetToken(UNKNOWNEND);
<Normal><!                            begin
                                        State := Comment;
                                        SetToken(MLCOMMENT);
                                      end;
<Normal><!\-\-                        begin
                                        State := CommentAlt;
                                        SetToken(MLCOMMENT);
                                      end;
<Normal><ident                       begin
                                        State := UnknownAttrs;
                                        SetToken(KEYWORD); // SetToken(UNKNOWN);
                                      end;
<Normal><ident:ident              begin
                                        State := UnknownAttrs;
                                        SetToken(KEYWORD); // SetToken(UNKNOWNPSEUDO);
                                      end;
<Normal><\/ident>?                  begin
                                        State := Normal;
                                        SetToken(KEYWORD); // SetToken(UNKNOWNEND);
                                      end;
<Normal><\/ident:ident>           begin
                                        State := Normal;
                                        SetToken(KEYWORD); // SetToken(UNKNOWNPSEUDOEND);
                                      end;

<UnknownAttrs>[ \t\n\r]+              SetToken(WHITESPACE);
<UnknownAttrs>ident                 SetToken(NAMETOKEN);
<UnknownAttrs>=                       begin
                                        SetToken(SYMBOL);
                                        State := AttrParm;
                                      end;
<UnknownAttrs>>                       begin
                                        State := Normal;
                                        SetToken(KEYWORD);
                                      end;
<UnknownAttrs>[^ \t\n\r=>]+           SetToken(UNKNOWNPARM);

<AttrParm>[ \t\n\r]+                  SetToken(WHITESPACE);
<AttrParm>[0-9]+%?                    |
<AttrParm>\"[0-9]+%?\"                |
<AttrParm>#[0-9a-fA-F]+               |
<AttrParm>\"#[0-9a-fA-F]+\"           |
<AttrParm>[0-9]+nmchar+             |
<AttrParm>\"[0-9]+nmchar+\"         begin
                                        SetToken(INTEGERNUMBER);
                                        State := UnknownAttrs;
                                      end;
<AttrParm>[a-zA-Z]{4,6}:\/\/[^>"]+    |
<AttrParm>\"[a-zA-Z]{3,6}:\/\/[^>"]+\"?   begin
                                            SetToken(URLSTRING);
                                            State := UnknownAttrs;
                                          end;
<AttrParm>(\"\")|('')                 |
<AttrParm>'[^>']+'                    |
<AttrParm>\"[^>"]+\"?                 begin
                                        State := UnknownAttrs;
                                        SetToken(STRINGCONSTANT);
                                      end;
<AttrParm>[^ ">\t\n\r]+               begin
                                        State := UnknownAttrs;
                                        SetToken(NAMETOKEN);
                                      end;

<Comment>[Dd][Oo][Cc][Tt][Yy][Pp][Ee][^>]+ SetToken(DOCTYPE);
<Comment>[^>]+                        SetToken(MLCOMMENT);
<Comment>>                            begin
                                        SetToken(MLCOMMENT);
                                        State := Normal;
                                      end;

<CommentAlt>[^>]+                     SetToken(MLCOMMENT);
<CommentAlt>>                         begin
                                        SetToken(MLCOMMENT);
                                        State := Normal;
                                      end;
<Script>.*<\/[Ss][Cc][Rr][Ii][Pp][Tt]> begin
                                         State := Normal;
                                         SetToken(SCRIPTPART);
                                       end;
<Script>.+                            SetToken(SCRIPTPART);
eof                                 SetToken(-1); // special token to mark input end

