unit EnvironmentOptionsDlg;

// 18-DEC-99 ml: bug fixes
// 05-DEC-99 ml: structural rework, scroll hint selection included
// 07-NOV-99 ml: MinimizeToTray option
// 04-JUL-99 ml: word wrap for font preview included
// 22-JUN-99 ml: implementation so far
//----------------------------------------------------------------------------------------------------------------------
// This unit contains a dialog to be used to choose options specific to the entire program.
// These options are stored in the registry under "HKEY_CURRENT_USER/Software/Delphi Compiler Generator".  

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, EditorKeyCommands,
  ColorPickerButton, SyntaxEditor;

type
  TEnvironmentOptionsDialog = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    PageControl: TPageControl;
    PreferenceSheet: TTabSheet;
    GroupBox1: TGroupBox;
    EditorCheckBox: TCheckBox;
    DesktopCheckBox: TCheckBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    DCCPathEdit: TEdit;
    DelphiRootButton: TSpeedButton;
    RegistrationSheet: TTabSheet;
    Label2: TLabel;
    Label3: TLabel;
    KeyCommandSheet: TTabSheet;
    KeyCmdList: TListView;
    EditButton: TButton;
    ResetButton: TButton;
    EditorSheet: TTabSheet;
    GroupBox3: TGroupBox;
    GroupUndoCheckBox: TCheckBox;
    AutoIndentCheckBox: TCheckBox;
    InsertCheckBox: TCheckBox;
    UseTabCheckBox: TCheckBox;
    SmartTabCheckBox: TCheckBox;
    OptimalFillCheckBox: TCheckBox;
    UnindentCheckBox: TCheckBox;
    ThroughTabsCheckBox: TCheckBox;
    BeyondEOLCheckBox: TCheckBox;
    UndoCheckBox: TCheckBox;
    BlanksCheckBox: TCheckBox;
    HighlightCheckBox: TCheckBox;
    UndoLimitEdit: TEdit;
    BlockIndentEdit: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    TabstopEdit: TEdit;
    Label6: TLabel;
    ColorSheet: TTabSheet;
    HighlighterComboBox: TComboBox;
    Label7: TLabel;
    AttributesListBox: TListBox;
    ControlCharsCheckBox: TCheckBox;
    FGColorButton: TColorPickerButton;
    BGColorButton: TColorPickerButton;
    GroupBox4: TGroupBox;
    BoldCheckBox: TCheckBox;
    ItalicCheckBox: TCheckBox;
    UnderlineCheckBox: TCheckBox;
    SampleEdit: TSyntaxEdit;
    DisplaySheet: TTabSheet;
    SpeedButton1: TSpeedButton;
    CursorROCheckBox: TCheckBox;
    DisplayGroupBox: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    MarkFGColorButton: TColorPickerButton;
    MarkBGColorButton: TColorPickerButton;
    Panel1: TPanel;
    FontComboBox: TComboBox;
    SizeComboBox: TComboBox;
    MultilineCheckBox: TCheckBox;
    IconCheckBox: TCheckBox;
    BackupCheckBox: TCheckBox;
    Bevel1: TBevel;
    RegButton1: TRadioButton;
    RegButton2: TRadioButton;
    RegButton3: TRadioButton;
    FileCheckBox1: TCheckBox;
    FileCheckBox2: TCheckBox;
    FileCheckBox3: TCheckBox;
    FileCheckBox4: TCheckBox;
    ScrollBarComboBox: TComboBox;
    Label11: TLabel;
    LineNumbersCheckBox: TCheckBox;
    FontSampleLabel: TLabel;
    GroupBox5: TGroupBox;
    MinimizeCheckBox: TCheckBox;
    ScrollHintCheckBox: TCheckBox;
    TripleClickCheckBox: TCheckBox;
    procedure DelphiRootButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EditButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ResetButtonClick(Sender: TObject);
    procedure HighlighterComboBoxChange(Sender: TObject);
    procedure AttributesListBoxClick(Sender: TObject);
    procedure BlockIndentEditExit(Sender: TObject);
    procedure UseTabCheckBoxClick(Sender: TObject);
    procedure SmartTabCheckBoxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FontComboBoxChange(Sender: TObject);
    procedure SizeComboBoxChange(Sender: TObject);
    procedure FontComboBoxMeasureItem(Control: TWinControl; Index: Integer; var Height: Integer);
    procedure FontComboBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure ColorButtonClick(Sender: TObject);
    procedure ColorButtonChange(Sender: TObject);
    procedure StyleCheckBoxClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure ColorButtonHint(Sender: TObject; Cell: Integer; var Hint: String);
    procedure MarkColorButtonClick(Sender: TObject);
    procedure MarkColorButtonChange(Sender: TObject);
    procedure RegButtonClick(Sender: TObject);
    procedure ScrollBarComboBoxChange(Sender: TObject);
  private
    FKeystrokes: TKeystrokes;
    FEditorOnly: Boolean;
    procedure SetKeystrokes(const Value: TKeystrokes);
    procedure UpdateKeystrokesList;
    procedure UpdateStyleBoxes(Styles: TFontStyles);
    procedure RegisterDCG;
    procedure UnregisterDCG;
  public
    property Keystrokes: TKeystrokes read FKeystrokes write SetKeystrokes;
    property EditorSettingsOnly: Boolean read FEditorOnly write FEditorOnly;
  end;

var EnvironmentOptionsDialog: TEnvironmentOptionsDialog;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  DCGStrings, DCGTools, DCGData, Dialogs, FileCtrl, ShortcutEditor, Menus, Main, Highlighter;

{$R *.DFM}

const
  DelphiLines =
    '{ Syntax Highlighting }'#13#10 +
      'procedure TForm1.Button1Click(Sender: TObject);'#13#10 +
      'var'#13#10 +
      '  Number, I, X: Integer;'#13#10 +
      'begin'#13#10 +
      '  Number := 12356;'#13#10 +
      '  X := 0;'#13#10 +
      '  Caption := ''The number is '' + IntToStr(Number);'#13#10 +
      '  asm'#13#10 +
      '    MOV AX,1234H'#13#10 +
      '    MOV Number,AX'#13#10 +
      '  end;'#13#10 +
      '  X := 10;'#13#10 +
      '  for I := 0 to Number do'#13#10 +
      '  begin'#13#10 +
      '    Inc(X);'#13#10 +
      '    Dec(X);'#13#10 +
      '    X := X + 1.0;'#13#10 +
      '    ListBox1.Items.Add(IntToStr(X));'#13#10 +
      '  end;'#13#10 +
      'end;';

    DCGLines =
      '{ Syntax Highlighting }'#13#10 +
      ' start state definitions'#13#10 +
      '%start Normal                 default state of lexer'#13#10 +
      '%start Comment                state started by { in normal state'#13#10 +
      ''#13#10 +
      ' macro definitions for rules'#13#10 +
      'letter    [A-Za-z_]'#13#10 +
      'digit     [0-9]'#13#10 +
      'identifier    letter(letter|digit)*'#13#10 +
      ''#13#10 +
      '<Normal>\[^]+\}'#13#10 +
      '  begin'#13#10 +
      '    Number := 12356;'#13#10 +
      '    Caption := ''The number is '' + IntToStr(Number);'#13#10 +
      '    asm'#13#10 +
      '      MOV AX,1234H'#13#10 +
      '      MOV Number,AX'#13#10 +
      '    end;'#13#10 +
      '  end;';

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.DelphiRootButtonClick(Sender: TObject);

var
  Path: String;

begin
  Path := ReplaceMacroByString(SDelphiPathMacro, DCCPathEdit.Text, DelphiRoot);
  if Pos('\\', Path) > 0 then Delete(Path, Pos('\\', Path), 1);

  with ApplicationData.OpenDialog do
  begin
    DefaultExt := '*.exe';
    FileName := Path;
    Filter := SExeFilter;

    Options := [ofEnableSizing, ofPathMustExist];
    if Execute then
    begin
      Path := ReplaceStringByMacro(SDelphiPathMacro, FileName, DelphiRoot);
      DCCPathEdit.Text := Path;
    end;
  end;

end;

//----------------------------------------------------------------------------------------------------------------------

procedure EnumFontCallback(lpelf: PEnumLogFont; lpntm: PNewTextMetric; FontType: Integer; CB: TComboBox); stdcall;

var
  Name: String[32];
  Temp: Integer;

begin
  // check whether we are enumerating font names or font sizes;
  // this can easily be done by checking the passed lParam as this is
  // one of the comboboxes
  if CB.Tag = 0 then
  begin
    // font enumeration
    if (lpelf.elfLogFont.lfPitchAndFamily and 3) = FIXED_PITCH then
    begin
      Name := lpelf.elfLogFont.lfFaceName;
      Temp := CB.Items.Add(Name);
      CB.Items.Objects[Temp] := Pointer(FontType);
    end;
  end
  else
  begin
    // size enumeration
    Temp := MulDiv(lpelf.elfLogFont.lfHeight, 72, Screen.PixelsPerInch);
    Name := Format('%3d', [Temp]);
    if CB.Items.IndexOf(Name) = -1 then CB.Items.Add(Name);
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FormActivate(Sender: TObject);

var
  I: Integer;
  DC: HDC;
  HL: TSyntaxHighlighter;

begin
  with GlobalSettings do
  begin
    // ----- preferences sheet
    // desktop options
    EditorCheckBox.Checked := AutoSaveEditor;
    DesktopCheckBox.Checked := AutoSaveDesktop;
    BackupCheckBox.Checked := CreateBackups;
    MinimizeCheckBox.Checked := MinimizeToTray;
    DCCPathEdit.Text := ReplaceStringByMacro(SDelphiPathMacro, GlobalSettings.DCC32Path, DelphiRoot);

    // ----- registration sheet
    // always set to "don't change anything"
    RegButton1.Checked := True;

    // ----- editor sheet
    // editor options
    GroupUndoCheckBox.Checked := eoGroupUndo in EditOptions;
    AutoIndentCheckBox.Checked := eoAutoIndent in EditOptions;
    InsertCheckBox.Checked := eoInserting in EditOptions;
    UseTabCheckBox.Checked := eoUseTabs in EditOptions;
    SmartTabCheckBox.Checked := eoSmartTabs in EditOptions;
    OptimalFillCheckBox.Checked := eoOptimalFill in EditOptions;
    UnindentCheckBox.Checked := eoAutoUnindent in EditOptions;
    ThroughTabsCheckBox.Checked := eoCursorThroughTabs in EditOptions;
    BeyondEOLCheckBox.Checked := eoScrollPastEOL in EditOptions;
    BlanksCheckBox.Checked := eoKeepTrailingBlanks in EditOptions;
    HighlightCheckBox.Checked := eoUseSyntaxHighlighting in EditOptions;
    UndoCheckBox.Checked := eoUndoAfterSave in EditOptions;
    ControlCharsCheckBox.Checked := eoShowControlChars in EditOptions;
    CursorROCheckBox.Checked := eoShowCursorWhileReadOnly in EditOptions;
    LineNumbersCheckBox.Checked := eoLineNumbers in EditOptions;
    ScrollHintCheckBox.Checked := eoShowScrollHint in EditOptions;
    TripleClickCheckBox.Checked := eoTripleClicks in EditOptions;
    UndoLimitEdit.Text := IntToStr(UndoLimit);
    BlockIndentEdit.Text := IntToStr(BlockIndent);
    TabStopEdit.Text := IntToStr(TabSize);

    // ----- display sheet
    MarkFGColorButton.SelectionColor := EditorMarkForeground;
    MarkBGColorButton.SelectionColor := EditorMarkBackground;
    ScrollBarComboBox.ItemIndex := Ord(ScrollBarMode);
    SampleEdit.MarkColor.Foreground := EditorMarkForeground;
    SampleEdit.MarkColor.Background := EditorMarkBackground;
    SampleEdit.ScrollBarMode := ScrollBarMode;
    MultilineCheckBox.Checked := MultilinePageControl;
    IconCheckBox.Checked := ShowFileIcons;

    // enumerate currently available fixed pitch fonts
    FontComboBox.Clear;
    DC := GetDC(0);
    try
      EnumFontFamilies(DC, nil, @EnumFontCallback, Longint(FontComboBox));
    finally
      ReleaseDC(0, DC);
    end;
    // select name and size of the currently active font
    FontComboBox.ItemIndex := FontComboBox.Items.IndexOf(EditorFontName);
    // Why the heck does the previous assignment not trigger OnChange?
    FontComboBoxChange(FontComboBox);
    SizeComboBox.ItemIndex := SizeComboBox.Items.IndexOf(Format('%3d', [EditorFontSize]));
    SizeComboBoxChange(SizeComboBox);

    // ----- color sheet
    // clear copies of highlighters if there are some
    for I := 0 to HighlighterComboBox.Items.Count - 1 do
      TSyntaxHighlighter(HighlighterComboBox.Items.Objects[I]).Free;
    HighlighterComboBox.Clear;
    with GlobalSettings do
    begin
      for I := 0 to HighlighterList.Count - 1 do
      begin
        // create a copy of each highlighter (note the polymorphic class creation, as this
        // is important to create the right instances)
        HL := HighlighterList[I];
        HL := THighlighterClass(HL.ClassType).Create(nil);
        HL.AssignAttributesFrom(TSyntaxHighlighter(HighlighterList[I]));
        HighlighterComboBox.Items.AddObject(HL.LanguageName, HL);
      end;
    end;
    
    ColorSheet.enabled := HighlighterComboBox.Items.Count > 0;
    if ColorSheet.enabled then
    begin
      HighlighterComboBox.ItemIndex := 0;
      HighlighterComboBoxChange(HighlighterComboBox);
    end;

    // ----- commands sheet
    // editor commands
    SetKeystrokes(EditorCommands);
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.UnregisterDCG;

// well, the user decided to remove all so do it now

var
  Key: HKEY;

begin
  // main key first
  if RegOpenKeyEx(HKEY_CURRENT_USER, PChar(SDCGRegistryKey), 0, KEY_ALL_ACCESS, Key) = ERROR_SUCCESS then
  begin
    EmptyRegistryKey(Key);
    RegCloseKey(Key);
    RegDeleteKey(HKEY_CURRENT_USER, PChar(SDCGRegistryKey));
  end;
                                          
  // file associations next                           
  RegDeleteKey(HKEY_CLASSES_ROOT, '.dcg');
  if RegOpenKeyEx(HKEY_CLASSES_ROOT, 'DCG.ProjectFile', 0, KEY_ALL_ACCESS, Key) = ERROR_SUCCESS then
  begin                                     
    EmptyRegistryKey(Key);
    RegCloseKey(Key);
    RegDeleteKey(HKEY_CLASSES_ROOT, 'DCG.ProjectFile');
  end;
  RegDeleteKey(HKEY_CLASSES_ROOT, '.l');
  if RegOpenKeyEx(HKEY_CLASSES_ROOT, 'DCG.LexerFile', 0, KEY_ALL_ACCESS, Key) = ERROR_SUCCESS then
  begin
    EmptyRegistryKey(Key);
    RegCloseKey(Key);
    RegDeleteKey(HKEY_CLASSES_ROOT, 'DCG.LexerFile');
  end;
  RegDeleteKey(HKEY_CLASSES_ROOT, '.y');
  if RegOpenKeyEx(HKEY_CLASSES_ROOT, 'DCG.ParserFile', 0, KEY_ALL_ACCESS, Key) = ERROR_SUCCESS then
  begin
    EmptyRegistryKey(Key);
    RegCloseKey(Key);
    RegDeleteKey(HKEY_CLASSES_ROOT, 'DCG.ParserFile');
  end;
  RegDeleteKey(HKEY_CLASSES_ROOT, '.cod');
  if RegOpenKeyEx(HKEY_CLASSES_ROOT, 'DCG.CodeFile', 0, KEY_ALL_ACCESS, Key) = ERROR_SUCCESS then
  begin
    EmptyRegistryKey(Key);
    RegCloseKey(Key);
    RegDeleteKey(HKEY_CLASSES_ROOT, 'DCG.CodeFile');
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.RegisterDCG;

// registers the file types and shell commands for all DCG files

var
  Key, SubKey: HKEY;
  Disposition: Integer;
  Entry,
  Association: String;

begin
  Association := '"' + Application.ExeName + '" "%1"';
  // project file (*.dcg)
  if FileCheckBox1.Checked then
  begin
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, '.dcg', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG.ProjectFile';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      RegCloseKey(Key);
    end;
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, 'DCG.ProjectFile', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'Delphi Compiler Generator Project';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      if RegCreateKeyEx(Key, 'DefaultIcon', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        Entry := Application.ExeName + ',1';
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Entry), Length(Entry));
        RegCloseKey(SubKey);
      end;
      if RegCreateKeyEx(Key, 'Shell\open\command', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Association), Length(Association));
        RegCloseKey(SubKey);
      end;
      RegCloseKey(Key);
    end;
  end;

  // lexer file (*.l)
  if FileCheckBox2.Checked then
  begin
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, '.l', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG.LexerFile';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      RegCloseKey(Key);
    end;
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, 'DCG.LexerFile', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG Lexical Description';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      if RegCreateKeyEx(Key, 'DefaultIcon', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        Entry := Application.ExeName + ',2';
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Entry), Length(Entry));
        RegCloseKey(SubKey);
      end;
      if RegCreateKeyEx(Key, 'Shell\open\command', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Association), Length(Association));
        RegCloseKey(SubKey);
      end;
      RegCloseKey(Key);
    end;
  end;

  // parser file (*.y)
  if FileCheckBox3.Checked then
  begin
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, '.y', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG.ParserFile';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      RegCloseKey(Key);
    end;
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, 'DCG.ParserFile', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG Parser Description';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      if RegCreateKeyEx(Key, 'DefaultIcon', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        Entry := Application.ExeName + ',3';
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Entry), Length(Entry));
        RegCloseKey(SubKey);
      end;
      if RegCreateKeyEx(Key, 'Shell\open\command', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Association), Length(Association));
        RegCloseKey(SubKey);
      end;
      RegCloseKey(Key);
    end;
  end;

  // code template file (*.cod)
  if FileCheckBox4.Checked then
  begin
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, '.cod', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG.CodeFile';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      RegCloseKey(Key);
    end;
    if RegCreateKeyEx(HKEY_CLASSES_ROOT, 'DCG.CodeFile', 0, nil, REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS, nil, Key, @Disposition) = ERROR_SUCCESS then
    begin
      Entry := 'DCG Code Template';
      RegSetValueEx(Key, '', 0, REG_SZ, PChar(Entry), Length(Entry));
      if RegCreateKeyEx(Key, 'DefaultIcon', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        Entry := Application.ExeName + ',4';
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Entry), Length(Entry));
        RegCloseKey(SubKey);
      end;
      if RegCreateKeyEx(Key, 'Shell\open\command', 0, nil, REG_OPTION_NON_VOLATILE,
                        KEY_ALL_ACCESS, nil, SubKey, @Disposition) = ERROR_SUCCESS then
      begin
        RegSetValueEx(SubKey, '', 0, REG_SZ, PChar(Association), Length(Association));
        RegCloseKey(SubKey);
      end;
      RegCloseKey(Key);
    end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.OKButtonClick(Sender: TObject);

var
  I: Integer;

begin
  if RegButton3.Checked then
  begin
    MessageBeep(MB_ICONQUESTION);
    if MessageDlg(SUnregister, mtConfirmation, [mbOK, mbCancel], 0) = mrOK then
    begin
      UnregisterDCG;
      GlobalSettings.DontWriteSettings := True;
      ModalResult := mrCancel;
    end;
    Exit;
  end;

  with GlobalSettings do
  begin
    // ----- preferences sheet
    // save options
    AutoSaveEditor := EditorCheckBox.Checked;
    AutoSaveDesktop := DesktopCheckBox.Checked;
    CreateBackups := BackupCheckBox.Checked;
    MinimizeToTray := MinimizeCheckBox.Checked;
    // registration
    if RegButton2.Checked then
    begin
      // register all DCG files
      GlobalSettings.DontWriteSettings := False;
      RegisterDCG;
    end;

    DCC32Path := ReplaceStringByMacro(SDelphiPathMacro, DCCPathEdit.Text, DelphiRoot);

    // ----- editor sheet
    if GroupUndoCheckBox.Checked then Include(EditOptions, eoGroupUndo)
                                 else Exclude(EditOptions, eoGroupUndo);
    if AutoIndentCheckBox.Checked then Include(EditOptions, eoAutoIndent)
                                  else Exclude(EditOptions, eoAutoIndent);
    if InsertCheckBox.Checked then Include(EditOptions, eoInserting)
                              else Exclude(EditOptions, eoInserting);
    if UseTabCheckBox.Checked then Include(EditOptions, eoUseTabs)
                              else Exclude(EditOptions, eoUseTabs);
    if SmartTabCheckBox.Checked then Include(EditOptions, eoSmartTabs)
                                else Exclude(EditOptions, eoSmartTabs);
    if OptimalFillCheckBox.Checked then Include(EditOptions, eoOptimalFill)
                                   else Exclude(EditOptions, eoOptimalFill);
    if UnindentCheckBox.Checked then Include(EditOptions, eoAutoUnindent)
                                else Exclude(EditOptions, eoAutoUnindent);
    if ThroughTabsCheckBox.Checked then Include(EditOptions, eoCursorThroughTabs)
                                   else Exclude(EditOptions, eoCursorThroughTabs);
    if BeyondEOLCheckBox.Checked then Include(EditOptions, eoScrollPastEOL)
                                 else Exclude(EditOptions, eoScrollPastEOL);
    if BlanksCheckBox.Checked then Include(EditOptions, eoKeepTrailingBlanks)
                              else Exclude(EditOptions, eoKeepTrailingBlanks);
    if HighlightCheckBox.Checked then Include(EditOptions, eoUseSyntaxHighlighting)
                                 else Exclude(EditOptions, eoUseSyntaxHighlighting);
    if UndoCheckBox.Checked then Include(EditOptions, eoUndoAfterSave)
                            else Exclude(EditOptions, eoUndoAfterSave);
    if ControlCharsCheckBox.Checked then Include(EditOptions, eoShowControlChars)
                                    else Exclude(EditOptions, eoShowControlChars);
    if CursorROCheckBox.Checked then Include(EditOptions, eoShowCursorWhileReadOnly)
                                else Exclude(EditOptions, eoShowCursorWhileReadOnly);
    if LineNumbersCheckBox.Checked then Include(EditOptions, eoLineNumbers)
                                   else Exclude(EditOptions, eoLineNumbers);
    if ScrollHintCheckBox.Checked then Include(EditOptions, eoShowScrollHint)
                                  else Exclude(EditOptions, eoShowScrollHint);
    if TripleClickCheckBox.Checked then Include(EditOptions, eoTripleClicks)
                                   else Exclude(EditOptions, eoTripleClicks);
    UndoLimit := StrToInt(UndoLimitEdit.Text);
    BlockIndent := StrToInt(BlockIndentEdit.Text);
    TabSize := StrToInt(TabStopEdit.Text);

    // ----- display sheet
    MultilinePageControl := MultilineCheckBox.Checked;
    ShowFileIcons := IconCheckBox.Checked;
    EditorMarkForeground := MarkFGColorButton.SelectionColor;
    EditorMarkBackground := MarkBGColorButton.SelectionColor;
    ScrollBarMode := TScrollBarMode(ScrollBarComboBox.ItemIndex);

    EditorFontName := FontComboBox.Items[FontComboBox.ItemIndex];
    if SizeComboBox.Items[SizeComboBox.ItemIndex] <> '' then
      EditorFontSize := StrToInt(SizeComboBox.Items[SizeComboBox.ItemIndex]);

    // ----- color sheet
    for I := 0 to HighlighterList.Count - 1 do
      TSyntaxHighlighter(HighlighterList[I]).AssignAttributesFrom(TSyntaxHighlighter(HighlighterComboBox.Items.Objects[I]));

    // ----- commands sheet
    EditorCommands.Assign(Keystrokes);
  end;

  ModalResult := mrOK;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FormCreate(Sender: TObject);

// initialize options dialog

begin
  with AttributesListBox do ControlStyle := ControlStyle + [csFramed];

  PageControl.ActivePage := PreferenceSheet;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.SetKeystrokes(const Value: TKeystrokes);

begin
  if FKeystrokes = nil then FKeystrokes := TKeystrokes.Create(Self);
  FKeystrokes.Assign(Value);
  UpdateKeystrokesList;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.UpdateKeystrokesList;

var
  I: Integer;

begin
  KeyCmdList.Items.BeginUpdate;
  try
    KeyCmdList.Items.Clear;
    for I := 0 to FKeystrokes.Count - 1 do
    begin
      with KeyCmdList.Items.Add do
      begin
        Data := Pointer(I);
        Caption := EditorCommandToDescrString(FKeystrokes[I].Command);
        if FKeystrokes[I].ShortCut = 0 then SubItems.Add('<none>')
                                       else SubItems.Add(Menus.ShortCutToText(FKeystrokes[I].ShortCut));
      end;
    end;
  finally
    KeyCmdList.Items.EndUpdate;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FormDestroy(Sender: TObject);

var
  I: Integer;

begin
  FKeystrokes.Free;
  for I := 0 to HighlighterComboBox.Items.Count - 1 do
    TSyntaxHighlighter(HighlighterComboBox.Items.Objects[I]).Free;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.EditButtonClick(Sender: TObject);

var
  SelItem: TListItem;
  OldShortcut: TShortcut;
  SCIndex: Integer;

begin
  SelItem := KeyCmdList.Selected;
  if SelItem = nil then Beep
                   else
    with TKeystrokeEditorForm.Create(Self) do
      try
        SCIndex := Integer(SelItem.Data);
        Command := FKeystrokes[SCIndex].Command;
        Keystroke := FKeystrokes[SCIndex].Shortcut;
        if ShowModal = mrOK then
        begin
          FKeystrokes[SCIndex].Command := Command;
          OldShortCut := FKeystrokes[SCIndex].ShortCut;
          try
            FKeystrokes[SCIndex].ShortCut := Keystroke;
          except
            on EKeyException do
              begin
                // Shortcut already exists in the collection!
                MessageDlg('The keystroke "' + Menus.ShortCutToText(Keystroke) +
                           '" is already assigned to another editor command.'#13#10 +
                           'The short cut for this item has not been changed.', mtError,
                           [mbOK], 0);
                FKeystrokes[SCIndex].ShortCut := OldShortCut;
              end;
            // Some other kind of exception, we don't deal with it...
          end;

          KeyCmdList.Items.BeginUpdate;
          try
            with SelItem do
            begin
              Caption := EditorCommandToDescrString(FKeystrokes[SCIndex].Command);
              if FKeystrokes[SCIndex].ShortCut = 0 then SubItems[0] := '<none>'
                                                   else SubItems[0] := Menus.ShortCutToText(FKeystrokes[SCIndex].Shortcut);
            end;
          finally
            KeyCmdList.Items.EndUpdate;
          end;
        end;
      finally
        Free;
      end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.ResetButtonClick(Sender: TObject);

begin
  FKeystrokes.ResetDefaults;
  UpdateKeystrokesList;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.HighlighterComboBoxChange(Sender: TObject);

// upates the list of attributes of the currently selected highlighter and other associated data

var
  HL: TSyntaxHighlighter;
  I: Integer;

begin
  AttributesListBox.Clear;
  HL := HighlighterComboBox.Items.Objects[HighlighterComboBox.ItemIndex] as TSyntaxHighlighter;
  SampleEdit.HighLighter := HL;
  for I := 0 to HL.AttributeCount - 1 do
    AttributesListBox.Items.Add(HL.Attribute[I].Name);

  if AttributesListBox.Items.Count > 0 then
  begin
    AttributesListBox.ItemIndex := 0;
    FGColorButton.Enabled := True;
    BGColorButton.Enabled := True;
    AttributesListBoxClick(nil);
  end
  else
  begin
    FGColorButton.Enabled := False;
    BGColorButton.Enabled := False;
  end;

  // set sample text
  if CompareText(HL.LanguageName, 'Delphi') = 0 then SampleEdit.Lines.Text := DelphiLines
                                                else
    if CompareText(HL.LanguageName, 'DCG') = 0 then SampleEdit.Lines.Text := DCGLines
                                               else SampleEdit.Lines.Text := 'no sample available';
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.AttributesListBoxClick(Sender: TObject);

// updates the color display of the currently selected highlighter attribute

var
  HL: TSyntaxHighlighter;

begin
  HL := HighlighterComboBox.Items.Objects[HighlighterComboBox.ItemIndex] as TSyntaxHighlighter;
  FGColorButton.SelectionColor := HL.Attribute[AttributesListBox.ItemIndex].Foreground;
  BGColorButton.SelectionColor := HL.Attribute[AttributesListBox.ItemIndex].Background;
  UpdateStyleBoxes(HL.Attribute[AttributesListBox.ItemIndex].Style);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.BlockIndentEditExit(Sender: TObject);

begin
  with Sender as TEdit do
  try
    StrToInt(Text);
  except
    SetFocus;
    raise;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.UseTabCheckBoxClick(Sender: TObject);

begin
  if UseTabCheckBox.Checked then SmartTabCheckbox.Checked := False;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.SmartTabCheckBoxClick(Sender: TObject);

begin
  if SmartTabCheckbox.Checked then UseTabCheckBox.Checked := False;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FormShow(Sender: TObject);

begin
  // display only sheets which are needed
  if FEditorOnly then
  begin
    PreferenceSheet.TabVisible := False;
    RegistrationSheet.TabVisible := False;       
  end
  else
  begin
    PreferenceSheet.TabVisible := True;
    RegistrationSheet.TabVisible := True;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FontComboBoxChange(Sender: TObject);

var
  DC: HDC;
  FontName: String;
  Size: String;
  I: Integer;

begin
  FontName := FontComboBox.Items[FontComboBox.ItemIndex];
  // keep last selected font size
  if SizeComboBox.ItemIndex > -1 then Size := SizeComboBox.Items[SizeComboBox.ItemIndex]
                                 else Size := ''; 
  SizeComboBox.Clear;
  if (Integer(FontComboBox.Items.Objects[FontComboBox.ItemIndex]) and RASTER_FONTTYPE) <> 0 then
  begin
    DC := GetDC(0);
    try
      EnumFontFamilies(DC, PChar(FontName), @EnumFontCallback, Longint(SizeComboBox));
    finally
      ReleaseDC(0, DC);
    end;
  end
  else
    for I := 7 to 30 do SizeComboBox.Items.Add(Format('%3d',[I]));

  FontSampleLabel.Font.Name := FontName;
  FontSampleLabel.Font.Pitch := fpFixed;
  SampleEdit.Font.Name := FontName;
  SampleEdit.Font.Pitch := fpFixed;

  // reselect last font size if available
  I := SizeComboBox.Items.IndexOf(Size);
  if I > -1 then SizeComboBox.ItemIndex := I
            else SizeComboBox.Text := Size;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.SizeComboBoxChange(Sender: TObject);

begin
  if Trim(SizeComboBox.Text) <> '' then
  begin
    FontSampleLabel.Font.Size := StrToInt(Trim(SizeComboBox.Text));
    SampleEdit.Font.Size := FontSampleLabel.Font.Size;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FontComboBoxMeasureItem(Control: TWinControl; Index: Integer; var Height: Integer);

begin
  if Index > -1 then
    with Canvas do
    begin
      Font.Name := TComboBox(Control).Items[Index];
      Font.Size := 10;
      Height := TextHeight('Wg') + 2; // measure ascenders and descenders
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.FontComboBoxDrawItem(Control: TWinControl; Index: Integer;
                                                         Rect: TRect; State: TOwnerDrawState);

begin
  if Index > -1 then
    with TComboBox(Control), TComboBox(Control).Canvas do
    begin
      FillRect(Rect);
      Font.Name := Items[Index];
      Font.Size := 10;
      TextOut(Rect.Left + 1, Rect.Top + 1, Items[Index]);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.ColorButtonClick(Sender: TObject);

// restores the foreground/background color of the currently selected
// attribute of the currently active highlighter
// Note: As this method uses the running index of the highlighter in both the
//       HighlighterCombobox as well as the global settings, neither should
//       be sorted, else the lists do not correspond correctly.

var
  HLOriginal,
  HLCopy: TSyntaxHighlighter;
  Index: Integer;

begin
  HLCopy := HighlighterComboBox.Items.Objects[HighlighterComboBox.ItemIndex] as TSyntaxHighlighter;
  HLOriginal := GlobalSettings.HighlighterList[HighlighterComboBox.ItemIndex];
  Index := AttributesListBox.ItemIndex;

  if Sender = FGColorButton then
  begin
    HLCopy.Attribute[Index].Foreground := HLOriginal.Attribute[Index].Foreground;
    FGColorButton.SelectionColor := HLOriginal.Attribute[Index].Foreground;
  end
  else
  begin
    HLCopy.Attribute[Index].Background := HLOriginal.Attribute[Index].Background;
    BGColorButton.SelectionColor := HLOriginal.Attribute[Index].Background;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.ColorButtonChange(Sender: TObject);

// updates the foreground/background color of the currently selected
// attribute of currently active highlighter

var
  HL: TSyntaxHighlighter;

begin
  HL := HighlighterComboBox.Items.Objects[HighlighterComboBox.ItemIndex] as TSyntaxHighlighter;
  with HL.Attribute[AttributesListBox.ItemIndex] do
    if Sender = FGColorButton then Foreground := FGColorButton.SelectionColor
                              else Background := BGColorButton.SelectionColor;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.StyleCheckBoxClick(Sender: TObject);

// updates the font style of the currently selected attribute of the
// currently active highlighter

var
  HL: TSyntaxHighlighter;
  NewStyles: TFontStyles;

begin
  HL := HighlighterComboBox.Items.Objects[HighlighterComboBox.ItemIndex] as TSyntaxHighlighter;
  with HL.Attribute[AttributesListBox.ItemIndex] do
  begin
    NewStyles := [];
    if BoldCheckBox.Checked then Include(NewStyles, fsBold);
    if ItalicCheckBox.Checked then Include(NewStyles, fsItalic);
    if UnderlineCheckBox.Checked then Include(NewStyles, fsUnderline);
    Style := NewStyles;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.SpeedButton1Click(Sender: TObject);

// restores the font stlyes of the currently selected
// attribute of the currently active highlighter
// Note: As this method uses the running index of the highlighter in both the
//       HighlighterCombobox as well as the global settings, neither should
//       be sorted, else the lists do not correspond correctly.

var
  HLOriginal,
  HLCopy: TSyntaxHighlighter;
  Index: Integer;

begin
  HLCopy := HighlighterComboBox.Items.Objects[HighlighterComboBox.ItemIndex] as TSyntaxHighlighter;
  HLOriginal := GlobalSettings.HighlighterList[HighlighterComboBox.ItemIndex];
  Index := AttributesListBox.ItemIndex;

  HLCopy.Attribute[Index].Style := HLOriginal.Attribute[Index].Style;
  UpdateStyleBoxes(HLCopy.Attribute[Index].Style);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.UpdateStyleBoxes(Styles: TFontStyles);

// sets or clears the font style checkboxes depending on the given styles

begin
  BoldCheckBox.Checked := fsBold in Styles;
  ItalicCheckBox.Checked := fsItalic in Styles;
  UnderlineCheckBox.Checked := fsUnderline in Styles;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.ColorButtonHint(Sender: TObject; Cell: Integer; var Hint: String);

begin
  case Cell of
    DefaultCell:
      Hint := 'Reset the color to its previous value';
    CustomCell:
      Hint := 'Compose your own color';
  else
    Hint := '';
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.MarkColorButtonClick(Sender: TObject);

// called when the user wants to reset one of the line mark's colors to its previous value

begin
  if Sender = MarkFGColorButton then MarkFGColorButton.SelectionColor := GlobalSettings.EditorMarkForeground
                                else MarkBGColorButton.SelectionColor := GlobalSettings.EditorMarkBackground;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.MarkColorButtonChange(Sender: TObject);

// called when the user has picked a mark line color

begin
  if Sender = MarkFGColorButton then SampleEdit.MarkColor.Foreground := MarkFGColorButton.SelectionColor
                                else SampleEdit.MarkColor.Background := MarkBGColorButton.SelectionColor;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.RegButtonClick(Sender: TObject);

begin
  FileCheckBox1.Enabled := RegButton2.Checked;
  FileCheckBox2.Enabled := RegButton2.Checked;
  FileCheckBox3.Enabled := RegButton2.Checked;
  FileCheckBox4.Enabled := RegButton2.Checked;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TEnvironmentOptionsDialog.ScrollBarComboBoxChange(Sender: TObject);

begin
  if ScrollbarComboBox.ItemIndex > -1 then
    SampleEdit.ScrollBarMode := TScrollBarMode(ScrollbarComboBox.ItemIndex);
end;

//----------------------------------------------------------------------------------------------------------------------

end.


