unit LexerProject;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ComCtrls, StdCtrls, ExtCtrls, SyntaxEditor, Highlighter,
  DelphiHighlighter, DCGData, LexerGenerator, Menus;

type
  TLexProjectForm = class(TBaseProjectForm)
    ResultEdit: TSyntaxEdit;
    DelphiHighlighter: TDelphiHighlighter;
    OpenDialog: TOpenDialog;
    PageControl: TPageControl;
    OptionSheet: TTabSheet;
    LexSourceButton: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    LexTargetButton: TSpeedButton;
    Label3: TLabel;
    LexTemplateButton: TSpeedButton;
    LexerSourcePathEdit: TEdit;
    LexerTargetPathEdit: TEdit;
    LexerCodePathEdit: TEdit;
    ResultSheet: TTabSheet;
    StartButton: TSpeedButton;
    LexerLog: TMemo;
    LexerSheet: TTabSheet;
    CodeSheet: TTabSheet;
    TabSheet3: TTabSheet;
    SaveButton: TSpeedButton;
    Label4: TLabel;
    LexerEdit: TSyntaxEdit;
    CodeEdit: TSyntaxEdit;
    StatusBar: TStatusBar;
    EditorPopup: TPopupMenu;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    SelectAllItem: TMenuItem;
    N1: TMenuItem;
    PropertiesItem: TMenuItem;
    Undo1: TMenuItem;
    Redo1: TMenuItem;
    N2: TMenuItem;
    procedure LexSourceButtonClick(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure LexTargetButtonClick(Sender: TObject);
    procedure LexTemplateButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormResize(Sender: TObject);
    procedure LexerSourcePathEditChange(Sender: TObject);
    procedure LexerSourcePathEditKeyPress(Sender: TObject; var Key: Char);
    procedure EditCaretChange(Sender: TObject; X, Y: Cardinal);
    procedure PageControlChange(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure ResultEditSettingChange(Sender: TObject);
    procedure EditorPopupPopup(Sender: TObject);
  private
    LexerGenerator: TLexerGenerator;
    FNewProject,
    FInitialShow,
    FRemote: Boolean;
    FCurrentEdit: TSyntaxEdit;
    procedure EnableCodeSourceEdit;
    procedure EnableLexerSourceEdit;
    procedure ValidateControls;
  public
    constructor CreateFromFile(AOwner: TComponent; const FileName: String; DoShow: Boolean; Remote: Boolean = False);

    procedure SetStatus(const Text: String); override;
    procedure Save; override;
    function SaveAs: Boolean; override;
    procedure RestoreLog; override;

    property CurrentEdit: TSyntaxEdit read FCurrentEdit; 
  end;

var
  LexProjectForm: TLexProjectForm;

//-----------------------------------------------------------------------------

implementation

uses Main;

{$R *.DFM}

//-----------------------------------------------------------------------------

procedure TLexProjectForm.LexSourceButtonClick(Sender: TObject);

begin
  with OpenDialog do
  begin
    DefaultExt := '*.l';
    FileName := LexerSourcePathEdit.Text;
    Filter := 'lexical description (*.l)|*.l';
    Options := [ofEnableSizing, ofExtensionDifferent, ofPathMustExist];
    if Execute then
    begin
      LexerSourcePathEdit.Text := FileName;
      EnableLexerSourceEdit;
    end;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.LexTargetButtonClick(Sender: TObject);

begin
  with OpenDialog do
  begin
    DefaultExt := '*.pas';
    FileName := LexerTargetPathEdit.Text;
    Filter := 'Delphi source file (*.pas)|*.pas';
    Options := [ofEnableSizing, ofPathMustExist, ofHideReadOnly, ofOverwritePrompt];
    if Execute then LexerTargetPathEdit.Text := FileName;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.LexTemplateButtonClick(Sender: TObject);

begin
  with OpenDialog do
  begin
    DefaultExt := '*.cod';
    FileName := LexerCodePathEdit.Text;
    Filter := 'Lexer template file (*.cod)|*.cod';
    Options := [ofEnableSizing, ofExtensionDifferent, ofPathMustExist];
    if Execute then
    begin
      LexerCodePathEdit.Text := FileName;
      EnableCodeSourceEdit;
    end;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.StartButtonClick(Sender: TObject);

begin
  ResultSheet.TabVisible := False;
  LexerGenerator := TLexerGenerator.Create(LexerSourcePathEdit.Text, LexerCodePathEdit.Text, LexerLog.Lines);
  LexerLog.Clear;
  Application.ProcessMessages;
  Lexergenerator.Execute;
  ResultEdit.Lines.Clear;
  ResultEdit.Lines.AddStrings(Lexergenerator.Target);
  LexerGenerator.Free;
  ResultSheet.TabVisible := True;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.FormCreate(Sender: TObject);

begin
  PageControl.ActivePage := OptionSheet;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.SaveButtonClick(Sender: TObject);

begin
  if LexerTargetPathEdit.Text <> '' then ResultEdit.Lines.SaveToFile(LexerTargetPathEdit.Text);
end;

//-----------------------------------------------------------------------------

constructor TLexProjectForm.CreateFromFile(AOwner: TComponent; const FileName: String; DoShow, Remote: Boolean);

  //----------------------------------------------

  function StringToBoolean(const Str: String): Boolean;

  begin
    Result:=(CompareText(Str,'True') = 0) or
            (CompareText(Str,'T') = 0) or
            (CompareText(Str,'Yes') = 0) or
            (CompareText(Str,'Y') = 0);
  end;

  //----------------------------------------------

var Settings : TStringList;

begin
  FInitialShow := DoShow;
  FRemote := Remote;
  Create(AOwner);
  BeginLoading;

  if FileName = '' then
  begin
    WorkingFile := 'new project.dcg';
    FNewProject := True;
  end
  else
  begin
    WorkingFile := FileName;
    FNewProject := False;
  end;

  // load project if available
  if FileExists(WorkingFile) then
  begin
    Settings := TStringList.Create;
    try
      Settings.LoadFromFile(WorkingFile);
      LexerSourcePathEdit.Text := Settings.Values['LexicalDescription'];
      LexerCodePathEdit.Text := Settings.Values['CodeTemplate'];
      LexerTargetPathEdit.Text := Settings.Values['TargetFile'];
      FNewProject:=False;
      ValidateControls;
    finally
      Settings.Free;
      NeedSave := False;
    end;
  end;
  EndLoading;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.RestoreLog;

begin

end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.Save;

var Settings: TStringList;

begin
  if FNewproject then SaveAs
                 else
  begin
    Settings:=TStringList.Create;
    try
      Settings.Values['LexicalDescription'] := LexerSourcePathEdit.Text;
      Settings.Values['CodeTemplate'] := LexerCodePathEdit.Text;
      Settings.Values['TargetFile'] := LexerTargetPathEdit.Text;
      Settings.SaveToFile(WorkingFile);
      NeedSave:=False;
    finally
      Settings.Free;
    end;
  end;
end;

//-----------------------------------------------------------------------------

function TLexProjectForm.SaveAs: Boolean;

begin
  with ApplicationData.SaveDialog do
  begin
    FileName := WorkingFile;
    DefaultExt := '*.dcg';
    Filter := 'Delphi Compiler Generator Project (*.dcg)|*.dcg';
    Options := [ofHideReadOnly, ofEnableSizing];
    Title := 'Select Delphi Compiler Generator Project:';
    if Execute then
    begin
      WorkingFile := FileName;
      Result := True;
      FNewProject := False;
      Save;
    end
    else Result:=False;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.SetStatus(const Text: String);

begin
  StatusBar.SimpleText := Text;
  if Text = '' then
  begin
    if PageControl.ActivePage <> OptionSheet then StatusBar.SimplePanel := False;
  end
  else
  begin
    StatusBar.SimplePanel := True;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);

// By passing CanClose = False I mark closing the window itself, which is not the same as
// closing the entire project (in which case changes have to be written out).

begin
  if not CanClose and NeedSave then
  begin
    MessageBeep(MB_ICONQUESTION);
    case MessageDlg('Save changes to ''' + Caption + '''?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes : if FNewProject then CanClose := SaveAs
                             else Save;
      mrNo  : CanClose := True;
    else
      CanClose := False;
    end;
  end
  else CanClose := True;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.FormResize(Sender: TObject);

begin
  PageControl.Height := Height - 70;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.LexerSourcePathEditChange(Sender: TObject);

begin
  NeedSave := True;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.EnableLexerSourceEdit;

begin
  LexerEdit.Lines.Clear;
  if FileExists(LexerSourcePathEdit.Text) then LexerEdit.Lines.LoadFromFile(LexerSourcePathEdit.Text);
  LexerSheet.TabVisible := True;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.EnableCodeSourceEdit;

begin
  CodeEdit.Lines.Clear;
  if FileExists(LexerCodePathEdit.Text) then CodeEdit.Lines.LoadFromFile(LexerCodePathEdit.Text);
  CodeSheet.TabVisible := True;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.LexerSourcePathEditKeyPress(Sender: TObject; var Key: Char);

begin
  if Key = #13 then
  begin
    Key := #0;
    case TComponent(Sender).Tag of
      0 : EnableLexerSourceEdit;
      1 : EnableCodeSourceEdit;
    end;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.ValidateControls;

begin
  if LexerSourcePathEdit.Text = '' then LexerSheet.TabVisible := False
                                   else EnableLexerSourceEdit;
  if LexerCodePathEdit.Text = '' then CodeSheet.TabVisible := False
                                 else EnableCodeSourceEdit;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.EditCaretChange(Sender: TObject; X, Y: Cardinal);

begin
  StatusBar.SimplePanel := False;
  StatusBar.Panels[0].Text := Format('%d: %d', [Y + 1, X + 1]);
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.PageControlChange(Sender: TObject);

begin
  case PageControl.ActivePage.TabIndex of
    0 : StatusBar.SimplePanel := True;
  else
    StatusBar.SimplePanel := False;
    case PageControl.ActivePage.TabIndex of
      1 : FCurrentEdit := LexerEdit;
      2 : FCurrentEdit := CodeEdit;
    else
      FCurrentEdit := ResultEdit;
    end;

    with FCurrentEdit do
    begin
      if Modified then StatusBar.Panels[1].Text := 'Modified'
                  else StatusBar.Panels[1].Text := '';
      if eoReadOnly  in Options then StatusBar.Panels[2].Text := 'Read only'
                                else
        if eoInserting in Options then StatusBar.Panels[2].Text := 'Insert'
                                  else StatusBar.Panels[2].Text := 'Overwrite';
    end;
  end;
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.EditChange(Sender: TObject);

begin
  NeedSave := True;
  StatusBar.Panels[1].Text := 'Modified'; 
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.ResultEditSettingChange(Sender: TObject);

begin
  with Sender as TSyntaxEdit do
    if eoInserting in Options then StatusBar.Panels[2].Text := 'Insert'
                              else StatusBar.Panels[2].Text := 'Overwrite';
end;

//-----------------------------------------------------------------------------

procedure TLexProjectForm.EditorPopupPopup(Sender: TObject);

begin

end;

//-----------------------------------------------------------------------------

end.





              
