unit ProjectOptionsDlg;

// 13-NOV-99 ml: structural rework
// 18-JUL-99 ml: relative pathes for files belonging to a project
// 20-JUN-99 ml: implementation so far
//----------------------------------------------------------------------------------------------------------------------
// This unit contains a dialog used to choose project specific options. These options are
// stored in the accompaning project file. Nothing special here...

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  DCGData, Buttons, ExtCtrls, ComCtrls, Project;

type
  TProjectOptionsDialog = class(TForm)
    OKButton: TButton;
    CancelBtn: TButton;
    PageControl: TPageControl;
    LexerSheet: TTabSheet;
    DelphiProjectSheet: TTabSheet;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    DelphiProjectButton: TSpeedButton;
    DelphiProjectPathEdit: TEdit;
    ParserSheet: TTabSheet;
    DelphiProjectCheckBox: TCheckBox;
    ParserCheckBox: TCheckBox;
    LexerCheckBox: TCheckBox;
    LexSourceButton: TSpeedButton;
    Label2: TLabel;
    Label3: TLabel;
    LexTargetButton: TSpeedButton;
    Label4: TLabel;
    LexTemplateButton: TSpeedButton;
    LexerSourcePathEdit: TEdit;
    LexerTargetPathEdit: TEdit;
    LexerCodePathEdit: TEdit;
    ParserSourceButton: TSpeedButton;
    Label5: TLabel;
    Label6: TLabel;
    ParserTargetButton: TSpeedButton;
    Label7: TLabel;
    ParserTemplateButton: TSpeedButton;
    ParserSourcePathEdit: TEdit;
    ParserTargetPathEdit: TEdit;
    ParserCodePathEdit: TEdit;
    procedure DelphiProjectButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure LexerSourcePathEditKeyPress(Sender: TObject; var Key: Char);
    procedure SourceButtonClick(Sender: TObject);
    procedure TemplateButtonClick(Sender: TObject);
    procedure TargetButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FProject: TProjectForm;
    function CompleteFileName(FileName: String): String;
  end;

var
  ProjectOptionsDialog: TProjectOptionsDialog;

function ShowProjectOptions(Project: TBaseProjectForm): Boolean;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  DCGStrings, DCGTools, Dialogs;

{$R *.DFM}

//----------------------------------------------------------------------------------------------------------------------

function ShowProjectOptions(Project: TBaseProjectForm): Boolean;

begin
  if Project is TProjectForm then
    with ProjectOptionsDialog do
    begin
      FProject := Project as TProjectForm;
      Caption := Format(SProjectPropertiesTitel, [PureName(FProject.WorkingFile)]);
      // fill edits
      with FProject, ProjectSettings do
      begin
        LexerCheckBox.Checked := GenerateLexer;
        LexerSourcePathEdit.Text := ExtractRelativePath(WorkingFile, LexerSource);
        LexerCodePathEdit.Text := ExtractRelativePath(WorkingFile, LexerTemplate);
        LexerTargetPathEdit.Text := ExtractRelativePath(WorkingFile, LexerTarget);
        ParserSourcePathEdit.Text := ExtractRelativePath(WorkingFile, ParserSource);

        ParserCheckBox.checked := GenerateParser;
        ParserCodePathEdit.Text := ExtractRelativePath(WorkingFile, ParserTemplate);
        ParserTargetPathEdit.Text := ExtractRelativePath(WorkingFile, ParserTarget);

        DelphiProjectCheckBox.checked := CompileDelphiProject;
        DelphiProjectPathEdit.Text := ExtractRelativePath(WorkingFile, DelphiProject);
      end;

      // finally show the form
      Result := ShowModal = mrOK;

      if Result then
        // update project settings
        with FProject.ProjectSettings do
        begin
          GenerateLexer := LexerCheckBox.checked;
          LexerSource := LexerSourcePathEdit.Text;
          LexerTemplate := LexerCodePathEdit.Text;
          LexerTarget := LexerTargetPathEdit.Text;
          ParserSource := ParserSourcePathEdit.Text;

          GenerateParser := ParserCheckBox.checked;
          ParserTemplate := ParserCodePathEdit.Text;
          ParserTarget := ParserTargetPathEdit.Text;

          CompileDelphiProject := DelphiProjectCheckBox.checked;
          DelphiProject := DelphiProjectPathEdit.Text;
        end;
    end
    else Result := False;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.DelphiProjectButtonClick(Sender: TObject);

begin
  with ApplicationData.OpenDialog do
  begin
    Title := SOpenTitel;
    FileName := DelphiProjectPathEdit.Text;
    Filter := SDelphiProjectFilter;
    Options := [ofEnableSizing, ofExtensionDifferent, ofPathMustExist];
    if Execute then DelphiProjectPathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.OKButtonClick(Sender: TObject);

begin
  FProject.ProjectSettings.DelphiProject := DelphiProjectPathEdit.Text;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.LexerSourcePathEditKeyPress(Sender: TObject; var Key: Char);

begin
  if Key = #13 then
  begin
    Key := #0;
    TEdit(Sender).SelectAll;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.SourceButtonClick(Sender: TObject);

begin
  with ApplicationData.OpenDialog do
  begin
    if TComponent(Sender).Tag = 0 then
    begin
      DefaultExt := '*.l';
      FileName := CompleteFileName(LexerSourcePathEdit.Text);
      Filter := SLexerFilter;
    end
    else
    begin
      DefaultExt := '*.y';
      FileName := CompleteFileName(ParserSourcePathEdit.Text);
      Filter := SParserFilter;
    end;
    Options := [ofEnableSizing, ofExtensionDifferent, ofPathMustExist];
    if Execute then
    begin
      if TComponent(Sender).Tag = 0 then
      begin
        LexerSourcePathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
        LexerSourcePathEdit.SetFocus;
      end
      else
      begin
        ParserSourcePathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
        ParserSourcePathEdit.SetFocus;
      end;
    end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.TemplateButtonClick(Sender: TObject);

begin
  with ApplicationData.OpenDialog do
  begin
    DefaultExt := '*.cod';
    Filter := SCodeTemplateFilter + '|' + SAnyFileFilter;
    if TComponent(Sender).Tag = 0 then
    begin
      FileName := CompleteFileName(LexerCodePathEdit.Text);
    end
    else
    begin
      FileName := CompleteFileName(ParserCodePathEdit.Text);
    end;

    Options := [ofEnableSizing, ofExtensionDifferent, ofPathMustExist];
    if Execute then
    begin
      if TComponent(Sender).Tag = 0 then
      begin
        LexerCodePathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
        LexerCodePathEdit.SetFocus;
      end
      else
      begin
        ParserCodePathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
        ParserCodePathEdit.SetFocus;
      end;
    end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.TargetButtonClick(Sender: TObject);

begin
  with ApplicationData.OpenDialog do
  begin
    DefaultExt := '*.pas';
    if TComponent(Sender).Tag = 0 then FileName := CompleteFileName(LexerTargetPathEdit.Text)
                                  else FileName := CompleteFileName(ParserTargetPathEdit.Text);
    Filter := SUnitFilter + '|' + SAnyFileFilter;
    Options := [ofEnableSizing, ofPathMustExist, ofHideReadOnly, ofOverwritePrompt];
    //InitialDir := ExtractFilePath(FileName);
    if Execute then
      if TComponent(Sender).Tag = 0 then
      begin
        LexerTargetPathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
        LexerTargetPathEdit.SetFocus;
      end
      else
      begin
        ParserTargetPathEdit.Text := ExtractRelativePath(FProject.WorkingFile, FileName);
        ParserTargetPathEdit.SetFocus;
      end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TProjectOptionsDialog.FormCreate(Sender: TObject);

begin
  PageControl.ActivePage := LexerSheet;
end;

//----------------------------------------------------------------------------------------------------------------------

function TProjectOptionsDialog.CompleteFileName(FileName: String): String;

// Returns FileName preceeded by the path of the project's base dir if it does not
// yet contain already a path. Otherwise FileName is just copied.

begin
  // a path name without a drive letter in it is assumed to be relative to the projects directory
  if (FileName = '') or
     (ExtractFileDrive(FileName) <> '') then Result := FileName
                                        else Result := ExtractFilePath(FProject.WorkingFile) + FileName;
end;

//----------------------------------------------------------------------------------------------------------------------

end.
