unit SyntaxRegister;

// Registration unit for the Unicode Syntax Edit control (USE). Most of the code is taken from mwEdit.

interface

procedure Register;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  DsgnIntf, Dialogs, Forms, Graphics, Classes,
  EditorKeyCommands,
  KeyCommandsEditor,
  DelphiHighlighter,
  CPPHighlighter,
  SQLHighlighter,
  HTMLHighlighter,
  DCGHighlighter,
  SyntaxEditor;

type
  TSyntaxEditorFontProperty = class(TFontProperty)
  public
    procedure Edit; override;
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure TSyntaxEditorFontProperty.Edit;

const
  // context ids for the Font editor 
  hcDFontEditor = 25000;

var FontDialog: TFontDialog;

begin
  FontDialog := TFontDialog.Create(Application);
  try
    FontDialog.Font := TFont(GetOrdValue);
    FontDialog.HelpContext := hcDFontEditor;
    FontDialog.Options := FontDialog.Options + [fdShowHelp, fdForceFontExist, fdFixedPitchOnly];
    if FontDialog.Execute then SetOrdValue(Longint(FontDialog.Font));
  finally
    FontDialog.Free;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

type
  TSyntaxEditorCommandProperty = class(TIntegerProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: String; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: String); override;
  end;

//----------------- TSyntaxEditorCommandProperty -----------------------------------------------------------------------

procedure TSyntaxEditorCommandProperty.Edit;

begin
  ShowMessage('I''m thinking that this will show a dialog that has a list'#13#10+
              'of all editor commands and a description of them to choose from.');
end;

//----------------------------------------------------------------------------------------------------------------------

function TSyntaxEditorCommandProperty.GetAttributes: TPropertyAttributes;

begin
  Result := [paMultiSelect, paDialog, paValueList, paRevertable];
end;

//----------------------------------------------------------------------------------------------------------------------

function TSyntaxEditorCommandProperty.GetValue: String;

begin
  Result := EditorCommandToCodeString(TEditorCommand(GetOrdValue));
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TSyntaxEditorCommandProperty.GetValues(Proc: TGetStrProc);

begin
  GetEditorCommandValues(Proc);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TSyntaxEditorCommandProperty.SetValue(const Value: String);

var NewValue: Longint;

begin
  if IdentToEditorCommand(Value, NewValue) then SetOrdValue(NewValue)
                                           else inherited SetValue(Value);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure Register;

begin
  RegisterComponents('Editor', [TSyntaxEdit, TDelphiHighlighter, TCPPHighlighter,
                                THTMLHighlighter, TSQLHighlighter, TDCGHighlighter]);
  RegisterPropertyEditor(TypeInfo(TFont), TSyntaxEdit, 'Font', TSyntaxEditorFontProperty);
  RegisterPropertyEditor(TypeInfo(TEditorCommand), nil, 'Command', TSyntaxEditorCommandProperty);
  RegisterPropertyEditor(TypeInfo(TKeystrokes), nil, '', TKeyStrokesProperty);
end;

//--------------------------------------------------------------------------------

end.
