 Description file for the Delphi Compiler Generator (DCG).
 This file contains definitions and rules for a lexical analyzer
 to tokenize DCG lexer and parser description files. Because these files
 contain mostly Delphi source code this lexer description is essentially an
 extended Delphi lexer description.


 ---------- first section (global definitions) ----------
LexerName DCGHighlighter      define name for the target module (unit name)
                              as well as for the lexer class itself (see also *.cod file)
Language 'DCG'
Filter 'Lexical description (*.l)|*.l|Parser description (*.y)|*.y|DCG code template (*.cod)|*.cod|'
%CaseSensitive  FALSE         tell DCG whether to use case sensitive keywords (allowed are
                              T and TRUE all other values are considered as false,
                              "CaseSensitive" might be abbreviated with "CS")

%%
 ---------- keyword section ----------
 all following identifiers (until section end) are considered as keywords
 this list will automatically be sorted
 1. reserved words
and
array
as
at
asm                      
begin
case
class
const
constructor
destructor
dispinterface
div
do
downto
else
end
except
exports
file
finalization
finally
for
function
goto
if
implementation
in
inherited
initialization
inline
interface
is
label
library
mod
nil
not
object
of
on
or
out
packed
procedure
program
property
raise
record
repeat
resourcestring
set
shl
shr
string
then
threadvar
to
try
type
unit
until
uses
var
while
with

 2. directives
absolute
abstract
assembler
automated
cdecl
contains
default
dispid
dynamic
export
external
far
forward
implements
index
message
name
near
nodefault
overload
override
package
pascal
private
protected
public
published
read
readonly
register
reintroduce
requires
resident
safecall
stdcall
stored
virtual
write
writeonly

%%
 ---------- definition section ----------
 token definitions
%T ASMTOKEN SLCOMMENT MLCOMMENT IDENTIFIER NUMBER WHITESPACE STRINGCONSTANT
%T SYMBOL UNKNOWN DCGKEYWORD DCGMACRO DCGCOMMENT

 start state definitions
%start Normal                 default state of lexer
%start StringState            string constants started by '
%start AssemblerState         state started by keyword asm
%start Comment                state started by { in normal state
%start Comment2               state started by { in assembler state
%start Comment3               state started by (* in normal state

 macro definitions for rules
letter        [A-Za-z_]
digit         [0-9]
hexdigit      [0-9A-Fa-f]
identifier    letter(letter|digit)*
integer       (digit+|\$hexdigit+)
sign          [\+\-]
exp           [Ee]sign?digit+
realinteger   digit+(((\.digit*)(exp)?)|exp)  note: a leading sign is not part of the number
white         [ \t]
symbol        [\(\)\-\#\$\&\*\+\,\.\/\:\;\<\=\>\@\[\]\^\%\!\?\~\\\\|]
eof           \0

%%
 ---------- rules section ----------

<Normal> |
<AssemblerState>'[^']*('|\n|\0)
  SetToken(STRINGCONSTANT);

<Normal>#?integer |
<Normal>realinteger
  SetToken(NUMBER);

 string constant enclosed in ""
<Normal>\"
  repeat
    case CurrentChar of
      '"',
      CR, #0:
        begin
          if CurrentChar = '"' then NextChar;
          SetToken(STRINGCONSTANT);
          Break;
        end;
      end;
      NextChar;
    until False;

 string constant enclosed in ''
<Normal>\'
  repeat
    case CurrentChar of
      '''',
      CR, #0:
        begin
          if CurrentChar = '''' then NextChar;
          SetToken(STRINGCONSTANT);
          Break;
        end;
      end;
    NextChar;
  until False;

 comment type //
<Normal>"//"
  begin
    while not (CurrentChar in [CR, #0]) do NextChar;
    SetToken(SLCOMMENT);
  end;

 comment type (**)
<Normal>"(*"
  repeat
    case CurrentChar of
      '*':
        if Lookahead = ')' then
        begin
          SetToken(MLCOMMENT);
          // skip star and closing parenthesis
          NextChar;
          NextChar;
          Break;
        end;
      CR, #0:
        begin
          State := Comment3;
          SetToken(MLCOMMENT);
          Break;
        end;
    end;
    NextChar;
  until False;

<Comment3>[^*]+
  SetToken(MLCOMMENT);

<Comment3>\*
  begin
    if CurrentChar = ')' then
    begin
      NextChar;
      State := Normal;
    end;
    SetToken(MLCOMMENT);
  end;

 comment type {}
<Normal>\{
  repeat
    case CurrentChar of
      '}',
      CR, #0:
        begin
          if CurrentChar = '}' then NextChar
                               else State := Comment;
          SetToken(MLCOMMENT);
        Break;
      end;
    end;
    NextChar;
  until False;

<Comment>[^}]+
  SetToken(MLCOMMENT);

<Comment>\}
  begin
    SetToken(MLComment);
    State := Normal;
  end;

<Normal>^%(%|identifier) |
<Normal>^<[^>]+>
  SetToken(DCGKEYWORD);

<Normal>\[^]+\
  SetToken(DCGMACRO);

<Normal>"".*
  SetToken(DCGCOMMENT);

<Normal>identifier
  if CompareText(GetToken, 'asm') = 0 then
  begin
    State := AssemblerState;
    IsKeyword;
  end
  else
    if not IsKeyword then SetToken(IDENTIFIER);

<AssemblerState>identifier
  if CompareText(GetToken, 'end') = 0 then
  begin
    IsKeyword;
    State := Normal;
  end
  else SetToken(ASMTOKEN);

<AssemblerState>\{
  repeat
    case CurrentChar of
      '}',
      CR, #0:
        begin
          if CurrentChar = '}' then NextChar
                               else State := Comment2;
          SetToken(MLCOMMENT);
          Break;
        end;
    end;
    NextChar;
  until False;
<Comment2>[^}]+
  SetToken(MLCOMMENT);
<Comment2>\}
  begin
    SetToken(MLComment);
    State := AssemblerState;
  end;

<AssemblerState>.
  SetToken(ASMTOKEN);

<Normal>white+
  SetToken(WHITESPACE);

 <Normal>symbol+
   SetToken(SYMBOL);

<Normal>.
  SetToken(SYMBOL);

eof
  SetToken(-1); // special token to mark input end

