 Template file for the Delphi Compiler Generator (DCG)
 this file contains definitions and rules for a lexical analyzer
 to tokenize DaCapo source code 
 
 Note: All identifiers which are lexer variables and/or definitions are case sensitive.
       Identifiers preceded by a "%" define lexer variables, all other define macros
       (except keywords of course).
 
 ---------- first section (global definitions) ----------
LexerName DaCapoHighlighter   define name for the target module (unit name)
                              as well as for the lexer class itself (see also Lexer.cod file)
Filter 'DaCapo files (*.dac)|*.dac'
%CaseSensitive  FALSE         tell DCG whether to use case sensitive keywords (allowed are
                              T and TRUE all other values are considered as false,
                              "CaseSensitive" might be abbrevated with "CS")
%%
 ---------- keyword section ----------
 all following identifiers (until section end) are considered as keywords
 this list will automatically be sorted
%%

%S Normal
%S Comment
%S Bold
%S Color1
%S Color2

letter                          [A-Za-z_]
digit                           [0-9]
identifier                      letter(letter|digit)*
integer                         digit+
sign                            [\+\-]
exp                             [Ee]sign?digit+
realinteger                     digit+(((\.digit*)(exp)?)|exp)
white                           [\1-\40]
eof                             \0

%%
<Normal>identifier  SetToken(IDENTIFIER);
<Normal>:digit+letter |
<Normal>~[A-Za-z]     |
<Normal>\\F           begin
                        SetToken(KEYWORD);
                      end;
<Normal>\"            repeat
                        case CurrentChar of
                          '"',
                          CR, #0 : begin
                                     if CurrentChar = '"' then NextChar;
                                     SetToken(STRINGCONSTANT);
                                     Break;
                                   end;
                          ':' : begin
                                  SetToken(STRINGCONSTANT);
                                  State := Comment;
                                  Break;
                                end;
                        end;
                        NextChar;
                      until False;
<Comment>\"           begin
                        SetToken(STRINGCONSTANT);
                        State := Normal;
                      end;
<Comment>:digit+letter begin
                             SetToken(KEYWORD);
                             if CurrentChar in [#0, CR] then State := Normal;
                           end;
<Comment>.            repeat
                        case CurrentChar of
                          '"',
                          CR, #0 : begin
                                     if CurrentChar = '"' then NextChar;
                                     SetToken(STRINGCONSTANT);
                                     State := Normal;
                                     Break;
                                   end;
                          ':' : begin
                                  SetToken(STRINGCONSTANT);
                                  Break;
                                end;
                        end;
                        NextChar;
                      until False;
<Normal>""           repeat
                        case CurrentChar of
                          '',
                          #0 : begin
                                 if CurrentChar = '' then NextChar
                                                      else State := Color1;
                                 SetToken(MLCOMMENT);
                                 Break;
                               end;
                        end;
                        NextChar;
                      until False;
<Color1>[^]*       SetToken(MLCOMMENT);
<Color1>""         begin
                        SetToken(MLCOMMENT);
                        State := Normal;
                      end;
<Normal>"\\_"         repeat
                        case CurrentChar of
                          '\'    : if Lookahead = '_' then
                                   begin
                                     // skip lookahead and break loop
                                     NextChar;
                                     NextChar;
                                     SetToken(MLCOMMENT);
                                     Break;
                                   end;
                          #0 : begin
                                 State := Color2;
                                 SetToken(MLCOMMENT);
                                 Break;
                               end;
                        end;
                        NextChar;
                      until False;
<Color2>[^\\_]*       begin
                        SetToken(MLCOMMENT);
                      end;
<Color2>\\            begin
                        SetToken(MLCOMMENT);
                        if CurrentChar = '_' then
                        begin
                          NextChar;
                          State := Normal;
                        end;
                      end;
<Normal>.             SetToken(Symbol);
eof                 SetToken(-1); // special token to mark input end

