 Template file for the Delphi Compiler Generator (DCG)
 this file contains definitions and rules for a lexical analyzer
 to tokenize Delphi source code (D4)
 
 Note: All identifiers which are lexer variables and/or definitions are case sensitive.
       Identifiers preceded by a "%" define lexer variables, all other define macros
       (except keywords of course).
 
 ---------- first section (global definitions) ----------
LexerName DelphiLexer         define name for the target module (unit name)
                              as well as for the lexer class itself (see also Lexer.cod file)
%CaseSensitive  FALSE         tell DCG whether to use case sensitive keywords (allowed are
                              T and TRUE all other values are considered as false,
                              "CaseSensitive" might be abbrevated with "CS") 
%%

 ---------- keyword section ----------
 all following identifiers (until section end) are considered as keywords
 this list will automatically be sorted

 1. reserved words
and
array
as
at
asm
begin
case
class
const
constructor
destructor
dispinterface
div
do
downto
else
end
except
exports
file
finalization
finally
for
function
goto
if
implementation
in
inherited
initialization
inline
interface
is
label
library
mod
nil
not
object
of
on
or
out
packed
procedure
program
property
raise
record
repeat
resourcestring
set
shl
shr
string
then
threadvar
to
try
type
unit
until
uses
var
while
with
 2. directives
absolute
abstract
assembler
automated
cdecl
contains
default
dispid
dynamic
export
external
far
forward
implements
index
message
name
near
nodefault
overload
override
package
pascal
private
protected
public
published
read
readonly
register
reintroduce
requires
resident
safecall
stdcall
stored
virtual
write
writeonly

%%
 ---------- definitions section ----------
 start state defintions
%Start Normal
%Start String1
%Start String2

%Start asmString1
%Start asmString2
%Start asmState

%Start OverreadState

%Start CompilerSwitch

%Start Comment

%Start CharTable

 macro definitions for rules
letter                          [A-Za-z_]
digit                           [0-9]
hexdigit                        [0-9A-Fa-f]
identifier                      letter(letter|digit)*
integer                         (digit+|\$hexdigit+)
sign                            [\+\-]
exp                             [Ee]sign?digit+
realinteger                     digit+(((\.digit*)(exp)?)|exp)
white                           [\1-\40]
eof                             \0

asmLabel                        @+(@|letter|digit)*
asmBinary                       [01]+[Bb]
asmOctal                        [0-7]+[Oo]
asmHex                          digithexdigit+[Hh]
asmWhite                        [\1-\11\13-\40]

%%
 ---------- rules section ----------
<Normal>'
          begin
            State := String2;
          end;
<Normal>"#"
          begin
            State :=  String1;
            //Unget_Char (yyText [1]);
          end;
<String1>'
          begin
            State :=  String2;
          end;
<String1>#integer
          begin   { #123 oder #$12 im String enthalten }
            Delete (yyText, 1, 1);
            Val (yyText, i, Result);
            if Result = 0 then
              s := s + Char (i)
            else begin
              SetToken (UNSIGNED_INTEGER);
              RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);
            end;
          end;
<String1>.
          begin
            State :=  (Normal);
            SetToken (STRING_CONST);
            UnGet_Char (yytext [1]);
            Input2Output (''''+s+'''');
            {RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);}
          end;
<String2>"''"
          s := s+'''';
<String2>'
          State :=  (String1);
<asmState>'
          begin
            State :=  (asmString1);
            s := '';
          end;
<asmString1>"''"
          s := s+'''';
<asmString1>'
          begin
            State :=  (asmState);
            SetToken (STRING_CONST);
            Input2Output (''''+s+'''');
          end;
<asmState>\"
          begin
            State :=  (asmString2);
            s := '';
          end;
<asmString2>""""
          s := s+'"';
<asmString2>\"
          begin
            State :=  (asmState);
            SetToken (STRING_CONST);
            Input2Output ('"'+s+'"');
          end;
<String2,asmString1,asmString2>\n
          begin
            RaiseScannerError (IDS_StringExceeedsLine, ['.']);  Halt;
          end;
<String2,asmString1,asmString2>.
          s := s + yyText;
<Normal>"(*"|"{"
          begin {State: Normal}
            EvalCommentStart(Normal);
          end;
<asmState>"(*"|"{"
          begin
            EvalCommentStart(asmState);
          end;
<asmState>identifier
          begin
            Input2Output (yyText);
            {if isAssemblerKeyword (yyText, KW) then
              SetToken (KW)
            else if isAssemblerRegister (yyText, KW) then
              SetToken (KW)
            else }
              SetToken (IDENTIFIER);
            if Upper (yyText) = 'end' then begin
              Return2OldState; {Could only be Normal}
              SetToken (_END_);
            end;
          end;
<asmState>asmLabel
          begin
            SetToken (ASM_LABEL);
            Input2Output (yytext);
          end;
<asmState>asmBinary
          begin
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, BinBase);
          end;
<asmState>asmOctal
          begin
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, OctBase);
          end;
<asmState>asmHex
          begin
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, HexBase);
          end;
<asmState>\n
          begin
            Input2Output( NEWLINE );
            SetToken (ASM_SEPERATOR);
          end;
<asmState>";"
          begin
            Input2Output (';');
            SetToken (ASM_SEPERATOR);
          end;
<asmState>asmWhite
          Input2Output (yyText);
<asmState>integer
          begin
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            if Result=0 then
              SetToken (UNSIGNED_INTEGER)
            else
              SetToken (ILLEGAL);
          end;
<asmState>.
          begin
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']
                               {'Illegal character: ', yyText});
            SetToken (ILLEGAL);
          end;
<Normal>realinteger/[^\.]
          begin {State: Normal}
            if (yyactchar = '.') then
              {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
              if (yylastchar = '.') then begin
                {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
                Val (copy(yyText,1,length(yytext)-1), yylVal.yyLongint, Result);
                Input2Output (copy(yyText,1,length(yytext)-1));
                if Result=0 then begin
                  SetToken (UNSIGNED_INTEGER);
                  unget_char( '.' );
                  exit;
                end else
                  SetToken (ILLEGAL);
              end else {z.B. '1.3e1..' oder '1.3..' }
                SetToken (ILLEGAL);
            {Normaler Real}
            Val (yyText, yylVal.yyExtended, Result);
            Input2Output (yyText);
            if Result=0 then begin
              SetToken (UNSIGNED_REAL);
            end else
              SetToken (ILLEGAL);
          end;
<Normal>integer
          begin {State: Normal}
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            if Result=0 then
              SetToken (UNSIGNED_INTEGER)
            else
              SetToken (ILLEGAL);
          end;
<Normal>identifier
          begin {State: Normal}
            Input2Output (yyText);
            if isKeyword(yyText, KW) then begin
              SetToken (KW);
              if KW = _ASM_ then
                SwitchState (Normal,asmState);
            end else if  isStdInstruction(yyText, KW) then
              if WouldAcceptChar(KW) then
                SetToken(KW)
              else
                SetToken (IDENTIFIER)
            else
              SetToken (IDENTIFIER);
          end;
<Normal>".."
          begin
            Input2Output (yyText);
            SetToken (DOTDOT);
          end;
<Normal>":="
          begin
            Input2Output (yyText);
            SetToken (_ASSIGNMENT_);
          end;
<Normal>"<="
          begin
            Input2Output (yyText);
            SetToken (LE);
          end;
<Normal>"<>"
          begin
            Input2Output (yyText);
            SetToken (NOTEQUAL);
          end;
<Normal>">="
          begin
            Input2Output (yyText);
            SetToken (GE);
          end;
<Normal>"(."
          begin
            Input2Output (yyText);
            yyText := '[';
            SetToken (LBRAC);
          end;
<Normal>".)"
          begin {State: Normal}
            Input2Output (yyText);
            yyText := ']';
            SetToken (RBRAC);
          end; 
<Normal>eof
          begin {State: Normal}
            Input2Output (yyText);
            SetToken (0);
          end;
<Normal>white
          begin {State: Normal}
            if yyText = #10 then
              Input2Output ( NEWLINE )
            else
              Input2Output (yyText);
          end;
<asmState,Normal>"+"
          SetToken(PLUS);
<asmState,Normal>"-"
          SetToken(MINUS);
<asmState,Normal>"*"
          SetToken(STAR);
<asmState,Normal>"/"
          SetToken(SLASH);
<asmState,Normal>"="
          SetToken(EQUAL);
<asmState,Normal>"<"
          SetToken(LT);
<asmState,Normal>">"
          SetToken(GT);
<asmState,Normal>","
          SetToken(COMMA);
<asmState,Normal>"("
          SetToken(LPAREN);
<asmState,Normal>")"
          SetToken(RPAREN);
<asmState,Normal>":"
          SetToken(COLON);
<asmState,Normal>";"
          SetToken(SEMICOLON);
<asmState,Normal>"@"
          SetToken(APE_SYMBOL);
<asmState,Normal>"^"
          SetToken(UPARROW);
<asmState,Normal>"["
          SetToken(LBRAC);
<asmState,Normal>"]"
          SetToken(RBRAC);
<asmState,Normal>"."
          SetToken(DOT);
<asmState>"&"
          SetToken(ET);
<Normal>.
          begin {State: Normal}
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']);
            SetToken (ILLEGAL);
          end;
<OverreadState>"(*"|"{"
          begin
            EvalCommentStart(OverreadState);
          end;
<OverreadState>\0
          begin
            Return2OldState; {Could be Normal or AsmState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          end;
<OverreadState>'
          begin
            s := '';
            if FormerState = Normal then
              SwitchState (OverreadState, String2)
            else if FormerState = asmState then
              SwitchState (OverreadState,asmString1)
            else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          end;
<OverreadState>\"
          begin
            if FormerState = Normal then
            else if FormerState = asmState then begin
              SwitchState (OverreadState,asmString2); s := '';
            end else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          end;
<OverreadState>.
          begin
            Input2Output (yyText);
          end;
<Comment>"*)"|"}"
          begin
            if ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) then begin
              Input2Output (yyText);
              Return2OldState; {Could be Normal or AsmState}
            end;
          end;
<Comment>\0
          begin
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            CommentEof;
          end;
<Comment>.
          begin
            Input2Output (yyText);
          end;
<CompilerSwitch>\0
          begin
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          end;
<CompilerSwitch>"*)"|"}"
          begin
            if ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) then begin
              Input2Output (yyText);
              {EvalCompilerSwitch( CompSwitch );}
              Return2OldState; {Could be Normal or AsmState or OverreadState}
            end;
          end;
<CompilerSwitch>.
          begin
            Input2Output (yyText);
            CompSwitch := CompSwitch + yyText ;
          end;
<CharTable>.
          begin
            Input2Output (yyText);
            s := yyText;
            SetToken (IMMCHAR_ID);
            Return2OldState;
          end;
