 Template file for the Delphi Compiler Generator (DCG)
 this file contains definitions and rules for a lexical analyzer
 to tokenize SQL source code 
 
 Note: All identifiers which are lexer variables and/or definitions are case sensitive.
       Identifiers preceded by a "%" define lexer variables, all other define macros
       (except keywords of course).
 
 ---------- first section (global definitions) ----------
LexerName SQLHighlighter      define name for the target module (unit name)
                              as well as for the lexer class itself (see also Lexer.cod file)
Filter 'SQL script files (*.sql)|*.sql'
%CaseSensitive  FALSE         tell DCG whether to use case sensitive keywords (allowed are
                              T and TRUE all other values are considered as false,
                              "CaseSensitive" might be abbrevated with "CS") 
%%
 ---------- keyword section ----------
 all following identifiers (until section end) are considered as keywords
 this list will automatically be sorted
active
add
all
allowed
after
alter
and
any
as
asc
ascending
at
audit
auto
autoinc
avg
base_name
before
begin
between
blob
boolean
both
by
bytes
cache
cast
char
character
check
check_point_length
close
collate
column
comment
commit
committed
composite
computed
conditional
constraint
containing
count
create
cstring
current
cursor
database
date
day
debug
dec
decimal
declare
default
delete
desc
descending
distinct
divideby
do
domain
double
drop
dump
else
end
entry_point
escape
exception
execute
exists
exit
external
extract
file
filter
float
for
foreign
fragment
from
full
function
gdscode
generator
gen_id
grant
group
group_commit_wait_time
having
hour
if
in
index
info
inactive
inner
input_type
insert
int
integer
intersect
into
is
isolation
join
key
leading
left
length
level
like
lock
logfile
long
lower
log_buffer_size
manual
max
maximum_segment
merge
message
min
minute
module_name
money
month
names
national
natural
nchar
no
not
null
num_log_buffers
numeric
of
on
only
open
option
or
order
outer
output_type
overflowpage_size
page
pages
parameter
password
plan
position
post_event
precision
procedure
protected
primary
privileges
program
raw_partitions
read
real
record_version
references
reserv
reserving
retain
returning_values
returns
revoke
right
rollback
rollforward
schema
second
segment
select
set
shared
shadow
show
singular
size
smallint
snapshot
some
sort
sqlcode
stability
starting
start
starts
statistics
store
sub_type
substring
sum
suspend
sync
synonym
table
then
time
timestamp
timezone_hour
timezone_minute
to
trailing
transaction
trigger
trim
type
uncommitted
union
unique
update
upper
user
value
values
varchar
variable
varying
view
wait
when
where
while
with
work
write
year

%%
 ---------- definitions section ----------

%T IDENTIFIER INTEGERNUMBER INTEGERNUMBER FLOATNUMBER WHITESPACE STRINGCONSTANT
%T MLCOMMENT SLCOMMENT SYMBOL KEYWORD UNKNOWN 

%S Normal
%S Comment1
%S Comment2

letter                          [A-Za-z_]
digit                           [0-9]
identifier                      letter(letter|digit)*
integer                         digit+
sign                            [\+\-]
exp                             [Ee]sign?digit+
realinteger                     digit+(((\.digit*)(exp)?)|exp) 
white                           [\1-\40]
eof                             \0

%%
 ---------- rules section ----------
<Normal>integer               SetToken(INTEGERNUMBER);
<Normal>realinteger           SetToken(FLOATNUMBER);
<Normal>white+                SetToken(WHITESPACE);
<Normal>identifier  if not IsKeyword then SetToken(IDENTIFIER);
<Normal>'             repeat
                        case CurrentChar of
                          ''''   : begin
                                     NextChar;
                                     if CurrentChar <> '''' then
                                     begin
                                       SetToken(STRINGCONSTANT);
                                       Break;
                                     end;
                                   end;
                          CR, #0 : begin
                                     SetToken(STRINGCONSTANT);
                                     Break;
                                   end;
                        end;
                        NextChar;
                      until False;
<Normal>\"            repeat
                        case CurrentChar of
                          '"',
                          CR, #0 : begin
                                     if CurrentChar = '"' then NextChar;
                                     SetToken(STRINGCONSTANT);
                                     Break;
                                   end;
                        end;
                        NextChar;
                      until False;
<Normal>"{"           repeat
                        case CurrentChar of
                          '}',
                          #0 : begin
                                 if CurrentChar = '}' then NextChar
                                                      else State := Comment1;
                                 SetToken(MLCOMMENT);
                                 Break;
                               end;
                        end;
                        NextChar;
                      until False;
<Comment1>[^}]*       SetToken(MLCOMMENT);
<Comment1>"}"         begin
                        SetToken(MLCOMMENT);
                        State := Normal;
                      end;
<Normal>"//"          repeat
                        case CurrentChar of
                          CR, #0 : begin
                                     SetToken(SLCOMMENT);
                                     Break;
                                   end;
                        end;
                        NextChar;
                      until False;
<Normal>"/*"          repeat
                        case CurrentChar of
                          '*'    : if Lookahead = '/' then
                                   begin
                                     // skip lookahead and break loop
                                     NextChar;
                                     NextChar;
                                     SetToken(MLCOMMENT);
                                     Break;
                                   end;
                          #0 : begin
                                 State := Comment2;
                                 SetToken(MLCOMMENT);
                                 Break;
                               end;
                        end;
                        NextChar;
                      until False;
<Comment2>[^\*]*      begin
                        SetToken(MLCOMMENT);
                      end;
<Comment2>\*          begin
                        SetToken(MLCOMMENT);
                        if CurrentChar = '/' then
                        begin
                          NextChar;
                          State := Normal;
                        end;
                      end;
<Normal>.             SetToken(SYMBOL); // any other char not catch before
eof                 SetToken(-1); // special token to mark input end

