%{
(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*                                                                  *)
(* (c) rr, 9.9., 19.9.,                                             *)
(* ---------------------------------------------------------------- *)

unit Scanner;

interface

{$R Scanner.Res}

%}

%Start Normal
%Start String1
%Start String2

%Start asmString1
%Start asmString2
%Start asmState

%Start OverreadState

%Start CompilerSwitch

%Start Comment

%Start CharTable

letter                          [A-Za-z_]
digit                           [0-9]
hexdigit                        [0-9A-Fa-f]
identifier                      {letter}({letter}|{digit})*
integer                         ({digit}+|\${hexdigit}+)
sign                            [\+\-]
exp                             [Ee]{sign}?{digit}+
realinteger                     {digit}+(((\.{digit}*)({exp})?)|{exp})
white                           [\1-\40]
eof                             \0

asmLabel                        @+(@|{letter}|{digit})*
asmBinary                       [01]+[Bb]
asmOctal                        [0-7]+[Oo]
asmHex                          {digit}{hexdigit}+[Hh]
asmWhite                        [\1-\11\13-\40]

%%

  implementation
  uses SysUtils, Parser, Errors, LexLib, YaccLib, ScanStack, CompSwitches;

  {$I Scanner.inc}

  CONST
    HexBase = 16;
    DecBase = 10;
    BinBase = 2;
    OctBase = 8;

  PROCEDURE RaiseScannerError( s : word; const Args : array of const );
    BEGIN
      writeln( LoadStr( s ) );
      { besser :
      raise ECompilerError.CreateError( @TTextRec(yyinput).Name, yylineno, yycolno, IDEM_Lex, s, Args );
      }
    END;

  PROCEDURE MakeInt (S: STRING; FixBase: BYTE);
    CONST
      Values: ARRAY [0..15] OF CHAR =
                ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                 'A', 'B', 'C', 'D', 'E', 'F');
    VAR
      i:     INTEGER;
      v:     LONGINT;
      Error: BOOLEAN;
      Base:  LONGINT;


    FUNCTION Index (c: CHAR): BYTE;
      VAR j, i: BYTE;
    BEGIN
      j := 16;
      i := 0;
      FOR i := 0 TO 15 DO
        IF UpCase (c) = Values [i] THEN
          j := i;
      IF j > 15 THEN
        Error := TRUE;     { Zeichen ungltig ! }
      Index := j;
    END;

  BEGIN
    Error := FALSE;
    Base := FixBase;
    v := Index (s [Length (s)]) * 1;
    FOR i := Length (S)-1 DOWNTO 1 DO BEGIN
      v := v + Index (s [i]) * Base;
      Base := Base * FixBase;
    END;
    IF NOT Error THEN BEGIN
      yylVal.yyLongint := v;
      Return (UNSIGNED_INTEGER);
    END ELSE BEGIN
      RaiseScannerError (IDS_NumberTooBig, ['.']);
      Return (ILLEGAL);
    END;
  END;

  PROCEDURE Commenteof;
  BEGIN
    RaiseScannerError (IDS_UnexpectedEOFInsideComment, [yyLineNo,'.']
                       {'Unexpected EOF inside Comment at line ', yyLineNo} );
  END;

  FUNCTION Upper (Str: STRING): STRING;
    VAR i: INTEGER;
  BEGIN
    FOR i := 1 TO Length (str) DO
      str [i] := UpCase (str [i]);
    Upper := Str;
  END;

  FUNCTION isKeyword (ID: STRING; VAR Token: INTEGER): BOOLEAN;
    (* checks whether id is Pascal keyword; if so, returns corresponding
       token number in token *)
      (* table of Pascal keywords: *)

  {$I Keywords.inc}

    begin
      id := UpperCase(id);
      isKeyword := kwlookup( id, token );
    end(*isKeyword*);

  function isStdInstruction(id : string; var token : integer) : boolean;
    (* checks whether id is Pascal keyword; if so, returns corresponding
       token number in token *)
     (* table of Pascal Standard Instructions: *)

  {$I Std_Inst.inc}

  var res : boolean;
    begin
      id := UpperCase(id); {$B-}
      isStdInstruction := kwlookup( id, token ){ and Parser.WouldAcceptChar( token )};
    end(*isStdInstruction*);

  var CompSwitch : string;

  Const NEWLINE : string[2] = #13#10;
  Procedure Input2Output( s : string );
    BEGIN
      {$ifdef yydebug}
      write( s );
      {$endif}
    END;

      (* {digit}+(((\.{digit}+)([Ee][+-]?{digit}+)?)|([Ee][+-]?{digit}+))
       {D}+(\.{D}* )?([Ee][+-]?{D}+)?
      {digit}+(\.{digit}* )?([Ee][+-]?{digit}+)? *)


  VAR
    Result:  INTEGER;
    c:       CHAR;
    KW:      INTEGER;
    s:       STRING;
    i:       INTEGER;
    cs:      STRING [2];

 Procedure EvalCommentStart( oldState: integer);
 var c : char;
  BEGIN
    cs := yyText;
    Input2Output (yyText);
    c := get_char;
    if c = '$' then begin
      SwitchState(oldState,CompilerSwitch);
      CompSwitch := '';
    end else begin
      SwitchState(oldState,Comment);
      Unget_Char(c);
    end;
  END;

<Normal>'
          BEGIN
            s := '';
            SwitchState (Normal,String2);{Start (String2);}
          END;
<Normal>"#"
          BEGIN
            s := '';
            SwitchState (Normal,String1);{Start (String1);}
            Unget_Char (yyText [1]);
          END;

<String1>'
          BEGIN
          Start (String2);
          END;
<String1>#{integer}
          BEGIN   { #123 oder #$12 im String enthalten }
            Delete (yyText, 1, 1);
            Val (yyText, i, Result);
            IF Result = 0 THEN
              s := s + Char (i)
            ELSE BEGIN
              Return (UNSIGNED_INTEGER);
              RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);
            END;
          END;
<String1>.
          BEGIN
            Return2OldState;{Start (Normal);}
            Return (STRING_CONST);
            UnGet_Char (yytext [1]);
            Input2Output (''''+s+'''');
            {RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);}
          END;

<String2>"''"
          s := s+'''';
<String2>'
          Start (String1);

<asmState>'
          BEGIN
            SwitchState (asmState,asmString1);{Start (asmString1);} s := '';
          END;
<asmString1>"''"
          s := s+'''';
<asmString1>'
          BEGIN
            Return2OldState;{Start (asmState);}
            Return (STRING_CONST);
            Input2Output (''''+s+'''');
          END;

<asmState>\"
          BEGIN
            SwitchState (asmState,asmString2);{Start (asmString2);} s := '';
          END;
<asmString2>""""
          s := s+'"';
<asmString2>\"
          BEGIN
            Return2OldState;{Start (asmState);}
            Return (STRING_CONST);
            Input2Output ('"'+s+'"');
          END;

<String2,asmString1,asmString2>\n
          BEGIN
            RaiseScannerError (IDS_StringExceeedsLine, ['.']);  Halt;
          END;
<String2,asmString1,asmString2>.
          s := s + yyText;



<Normal>"(*"|"{"
          BEGIN {State: Normal}
            EvalCommentStart(Normal);
          END;

<asmState>"(*"|"{"
          BEGIN
            EvalCommentStart(asmState);
          END;

<asmState>{identifier}
          BEGIN
            Input2Output (yyText);
            {IF isAssemblerKeyword (yyText, KW) THEN
              Return (KW)
            ELSE IF isAssemblerRegister (yyText, KW) THEN
              Return (KW)
            ELSE }
              Return (IDENTIFIER);
            IF Upper (yyText) = 'END' THEN BEGIN
              Return2OldState; {Could only be Normal}
              Return (_END_);
            END;
          END;
<asmState>{asmLabel}
          BEGIN
            Return (ASM_LABEL);
            Input2Output (yytext);
          END;
<asmState>{asmBinary}
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, BinBase);
          END;
<asmState>{asmOctal}
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, OctBase);
          END;
<asmState>{asmHex}
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, HexBase);
          END;
<asmState>\n
          BEGIN
            Input2Output( NEWLINE );
            Return (ASM_SEPERATOR);
          END;
<asmState>";"
          BEGIN
            Input2Output (';');
            Return (ASM_SEPERATOR);
          END;
<asmState>{asmWhite}
          Input2Output (yyText);
<asmState>{integer}
          BEGIN
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            IF Result=0 THEN
              Return (UNSIGNED_INTEGER)
            ELSE
              Return (ILLEGAL);
          END;
<asmState>.
          BEGIN
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']
                               {'Illegal character: ', yyText});
            Return (ILLEGAL);
          END;
<Normal>{realinteger}/[^\.]
          BEGIN {State: Normal}             
            if (yyactchar = '.') then
              {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
              if (yylastchar = '.') then begin
                {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
                Val (copy(yyText,1,length(yytext)-1), yylVal.yyLongint, Result);
                Input2Output (copy(yyText,1,length(yytext)-1));
                IF Result=0 THEN BEGIN
                  Return (UNSIGNED_INTEGER);
                  unget_char( '.' );
                  exit;
                end else
                  Return (ILLEGAL);
              end else {z.B. '1.3e1..' oder '1.3..' }
                Return (ILLEGAL);
            {Normaler Real}
            Val (yyText, yylVal.yyExtended, Result);
            Input2Output (yyText);
            IF Result=0 THEN BEGIN
              Return (UNSIGNED_REAL);
            END ELSE
              Return (ILLEGAL);
          END;
<Normal>{integer}
          BEGIN {State: Normal}
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            IF Result=0 THEN
              Return (UNSIGNED_INTEGER)
            ELSE
              Return (ILLEGAL);
          END;
<Normal>{identifier}
          BEGIN {State: Normal}
            Input2Output (yyText);
            IF isKeyword(yyText, KW) THEN BEGIN
              Return (KW);
              IF KW = _ASM_ THEN
                SwitchState (Normal,asmState);
            END ELSE IF  isStdInstruction(yyText, KW) THEN
              if WouldAcceptChar(KW) then
                Return(KW)
              else
                Return (IDENTIFIER)
            ELSE
              Return (IDENTIFIER);
          END;
<Normal>".."
          BEGIN
            Input2Output (yyText);
            Return (DOTDOT);
          END;
<Normal>":="
          BEGIN
            Input2Output (yyText);
            Return (_ASSIGNMENT_);
          END;
<Normal>"<="
          BEGIN
            Input2Output (yyText);
            Return (LE);
          END;
<Normal>"<>"
          BEGIN
            Input2Output (yyText);
            Return (NOTEQUAL);
          END;
<Normal>">="
          BEGIN
            Input2Output (yyText);
            Return (GE);
          END;
<Normal>"(."
          BEGIN
            Input2Output (yyText);
            yyText := '[';
            Return (LBRAC);
          END;
<Normal>".)"
          BEGIN {State: Normal}
            Input2Output (yyText);
            yyText := ']';
            Return (RBRAC);
          END; 
<Normal>{eof}
          BEGIN {State: Normal}
            Input2Output (yyText);
            Return (0);
          END;
<Normal>{white}
          BEGIN {State: Normal}
            IF yyText = #10 THEN
              Input2Output ( NEWLINE )
            ELSE
              Input2Output (yyText);
          END;

<asmState,Normal>"+"
          Return(PLUS);
<asmState,Normal>"-"
          Return(MINUS);
<asmState,Normal>"*"
          Return(STAR);
<asmState,Normal>"/"
          Return(SLASH);
<asmState,Normal>"="
          Return(EQUAL);
<asmState,Normal>"<"
          Return(LT);
<asmState,Normal>">"
          Return(GT);
<asmState,Normal>","
          Return(COMMA);
<asmState,Normal>"("
          Return(LPAREN);
<asmState,Normal>")"
          Return(RPAREN);
<asmState,Normal>":"
          Return(COLON);
<asmState,Normal>";"
          Return(SEMICOLON);
<asmState,Normal>"@"
          Return(APE_SYMBOL);
<asmState,Normal>"^"
          Return(UPARROW);
<asmState,Normal>"["
          Return(LBRAC);
<asmState,Normal>"]"
          Return(RBRAC);
<asmState,Normal>"."
          Return(DOT);
<asmState>"&"
          Return(ET);

<Normal>.
          BEGIN {State: Normal}
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']);
            Return (ILLEGAL);
          END;

<OverreadState>"(*"|"{"
          BEGIN
            EvalCommentStart(OverreadState);
          END;
<OverreadState>\0
          BEGIN
            Return2OldState; {Could be Normal or AsmState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          END;
<OverreadState>'
          BEGIN
            s := '';
            if FormerState = Normal then
              SwitchState (OverreadState, String2)
            else if FormerState = asmState then
              SwitchState (OverreadState,asmString1)
            else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          END;
<OverreadState>\"
          BEGIN
            if FormerState = Normal then
            else if FormerState = asmState then begin
              SwitchState (OverreadState,asmString2); s := '';
            end else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          END;
<OverreadState>.
          BEGIN
            Input2Output (yyText);
          END;


<Comment>"*)"|"}"
          BEGIN
            IF ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) THEN BEGIN
              Input2Output (yyText);
              Return2OldState; {Could be Normal or AsmState}
            END;
          END;
<Comment>\0
          BEGIN
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            CommentEof;
          END;
<Comment>.
          BEGIN
            Input2Output (yyText);
          END;


<CompilerSwitch>\0
          BEGIN
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          END;
<CompilerSwitch>"*)"|"}"
          BEGIN
            IF ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) THEN BEGIN
              Input2Output (yyText);
              {EvalCompilerSwitch( CompSwitch );}
              Return2OldState; {Could be Normal or AsmState or OverreadState}
            END;
          END;
<CompilerSwitch>.
          BEGIN
            Input2Output (yyText);
            CompSwitch := CompSwitch + yyText ;
          END;

<CharTable>.
          BEGIN
            Input2Output (yyText);
            s := yyText;
            Return (IMMCHAR_ID);
            Return2OldState;           
          END;
