// lexical analyzer template 
(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*                                                                  *)
(* (c) rr, 9.9., 19.9.,                                             *)
(* ---------------------------------------------------------------- *)

unit Scanner;

interface

{$R Scanner.Res}


const Normal = 2;
const String1 = 4;
const String2 = 6;

const asmString1 = 8;
const asmString2 = 10;
const asmState = 12;

const OverreadState = 14;

const CompilerSwitch = 16;

const Comment = 18;

const CharTable = 20;



function yylex : Integer;

  procedure yyaction(Rule: Integer);

  implementation
  uses SysUtils, Parser, Errors, LexLib, YaccLib, ScanStack, CompSwitches;

  {$I Scanner.inc}

  CONST
    HexBase = 16;
    DecBase = 10;
    BinBase = 2;
    OctBase = 8;

  PROCEDURE RaiseScannerError( s : word; const Args : array of const );
    BEGIN
      writeln( LoadStr( s ) );
      { besser :
      raise ECompilerError.CreateError( @TTextRec(yyinput).Name, yylineno, yycolno, IDEM_Lex, s, Args );
      }
    END;

  PROCEDURE MakeInt (S: STRING; FixBase: BYTE);
    CONST
      Values: ARRAY [0..15] OF CHAR =
                ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                 'A', 'B', 'C', 'D', 'E', 'F');
    VAR
      i:     INTEGER;
      v:     LONGINT;
      Error: BOOLEAN;
      Base:  LONGINT;


    FUNCTION Index (c: CHAR): BYTE;
      VAR j, i: BYTE;
    BEGIN
      j := 16;
      i := 0;
      FOR i := 0 TO 15 DO
        IF UpCase (c) = Values [i] THEN
          j := i;
      IF j > 15 THEN
        Error := TRUE;     { Zeichen ungltig ! }
      Index := j;
    END;

  BEGIN
    Error := FALSE;
    Base := FixBase;
    v := Index (s [Length (s)]) * 1;
    FOR i := Length (S)-1 DOWNTO 1 DO BEGIN
      v := v + Index (s [i]) * Base;
      Base := Base * FixBase;
    END;
    IF NOT Error THEN BEGIN
      yylVal.yyLongint := v;
      Return (UNSIGNED_INTEGER);
    END ELSE BEGIN
      RaiseScannerError (IDS_NumberTooBig, ['.']);
      Return (ILLEGAL);
    END;
  END;

  PROCEDURE Commenteof;
  BEGIN
    RaiseScannerError (IDS_UnexpectedEOFInsideComment, [yyLineNo,'.']
                       {'Unexpected EOF inside Comment at line ', yyLineNo} );
  END;

  FUNCTION Upper (Str: STRING): STRING;
    VAR i: INTEGER;
  BEGIN
    FOR i := 1 TO Length (str) DO
      str [i] := UpCase (str [i]);
    Upper := Str;
  END;

  FUNCTION isKeyword (ID: STRING; VAR Token: INTEGER): BOOLEAN;
    (* checks whether id is Pascal keyword; if so, returns corresponding
       token number in token *)
      (* table of Pascal keywords: *)

  {$I Keywords.inc}

    begin
      id := UpperCase(id);
      isKeyword := kwlookup( id, token );
    end(*isKeyword*);

  function isStdInstruction(id : string; var token : integer) : boolean;
    (* checks whether id is Pascal keyword; if so, returns corresponding
       token number in token *)
     (* table of Pascal Standard Instructions: *)

  {$I Std_Inst.inc}

  var res : boolean;
    begin
      id := UpperCase(id); {$B-}
      isStdInstruction := kwlookup( id, token ){ and Parser.WouldAcceptChar( token )};
    end(*isStdInstruction*);

  var CompSwitch : string;

  Const NEWLINE : string[2] = #13#10;
  Procedure Input2Output( s : string );
    BEGIN
      {$ifdef yydebug}
      write( s );
      {$endif}
    END;

      (* {digit}+(((\.{digit}+)([Ee][+-]?{digit}+)?)|([Ee][+-]?{digit}+))
       {D}+(\.{D}* )?([Ee][+-]?{D}+)?
      {digit}+(\.{digit}* )?([Ee][+-]?{digit}+)? *)


  VAR
    Result:  INTEGER;
    c:       CHAR;
    KW:      INTEGER;
    s:       STRING;
    i:       INTEGER;
    cs:      STRING [2];

 Procedure EvalCommentStart( oldState: integer);
 var c : char;
  BEGIN
    cs := yyText;
    Input2Output (yyText);
    c := get_char;
    if c = '$' then begin
      SwitchState(oldState,CompilerSwitch);
      CompSwitch := '';
    end else begin
      SwitchState(oldState,Comment);
      Unget_Char(c);
    end;
  END;

  begin
    // actions
    case Rule of
         0 : 
          BEGIN
            s := '';
            SwitchState (Normal,String2);{Start (String2);}
          END;
         1 : 
          BEGIN
            s := '';
            SwitchState (Normal,String1);{Start (String1);}
            Unget_Char (yyText [1]);
          END;

         2 : 
          BEGIN
          Start (String2);
          END;
         3 : 
          BEGIN   { #123 oder #$12 im String enthalten }
            Delete (yyText, 1, 1);
            Val (yyText, i, Result);
            IF Result = 0 THEN
              s := s + Char (i)
            ELSE BEGIN
              Return (UNSIGNED_INTEGER);
              RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);
            END;
          END;
         4 : 
          BEGIN
            Return2OldState;{Start (Normal);}
            Return (STRING_CONST);
            UnGet_Char (yytext [1]);
            Input2Output (''''+s+'''');
            {RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);}
          END;

         5 : 
          s := s+'''';
         6 : 
          Start (String1);

         7 : 
          BEGIN
            SwitchState (asmState,asmString1);{Start (asmString1);} s := '';
          END;
         8 : 
          s := s+'''';
         9 : 
          BEGIN
            Return2OldState;{Start (asmState);}
            Return (STRING_CONST);
            Input2Output (''''+s+'''');
          END;

        10 : 
          BEGIN
            SwitchState (asmState,asmString2);{Start (asmString2);} s := '';
          END;
        11 : 
          s := s+'"';
        12 : 
          BEGIN
            Return2OldState;{Start (asmState);}
            Return (STRING_CONST);
            Input2Output ('"'+s+'"');
          END;

        13 : 
          BEGIN
            RaiseScannerError (IDS_StringExceeedsLine, ['.']);  Halt;
          END;
        14 : 
          s := s + yyText;



        15 : 
          BEGIN {State: Normal}
            EvalCommentStart(Normal);
          END;

        16 : 
          BEGIN
            EvalCommentStart(asmState);
          END;

        17 : 
          BEGIN
            Input2Output (yyText);
            {IF isAssemblerKeyword (yyText, KW) THEN
              Return (KW)
            ELSE IF isAssemblerRegister (yyText, KW) THEN
              Return (KW)
            ELSE }
              Return (IDENTIFIER);
            IF Upper (yyText) = 'END' THEN BEGIN
              Return2OldState; {Could only be Normal}
              Return (_END_);
            END;
          END;
        18 : 
          BEGIN
            Return (ASM_LABEL);
            Input2Output (yytext);
          END;
        19 : 
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, BinBase);
          END;
        20 : 
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, OctBase);
          END;
        21 : 
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, HexBase);
          END;
        22 : 
          BEGIN
            Input2Output( NEWLINE );
            Return (ASM_SEPERATOR);
          END;
        23 : 
          BEGIN
            Input2Output (';');
            Return (ASM_SEPERATOR);
          END;
        24 : 
          Input2Output (yyText);
        25 : 
          BEGIN
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            IF Result=0 THEN
              Return (UNSIGNED_INTEGER)
            ELSE
              Return (ILLEGAL);
          END;
        26 : 
          BEGIN
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']
                               {'Illegal character: ', yyText});
            Return (ILLEGAL);
          END;
        27 : 
          BEGIN {State: Normal}             
            if (yyactchar = '.') then
              {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
              if (yylastchar = '.') then begin
                {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
                Val (copy(yyText,1,length(yytext)-1), yylVal.yyLongint, Result);
                Input2Output (copy(yyText,1,length(yytext)-1));
                IF Result=0 THEN BEGIN
                  Return (UNSIGNED_INTEGER);
                  unget_char( '.' );
                  exit;
                end else
                  Return (ILLEGAL);
              end else {z.B. '1.3e1..' oder '1.3..' }
                Return (ILLEGAL);
            {Normaler Real}
            Val (yyText, yylVal.yyExtended, Result);
            Input2Output (yyText);
            IF Result=0 THEN BEGIN
              Return (UNSIGNED_REAL);
            END ELSE
              Return (ILLEGAL);
          END;
        28 : 
          BEGIN {State: Normal}
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            IF Result=0 THEN
              Return (UNSIGNED_INTEGER)
            ELSE
              Return (ILLEGAL);
          END;
        29 : 
          BEGIN {State: Normal}
            Input2Output (yyText);
            IF isKeyword(yyText, KW) THEN BEGIN
              Return (KW);
              IF KW = _ASM_ THEN
                SwitchState (Normal,asmState);
            END ELSE IF  isStdInstruction(yyText, KW) THEN
              if WouldAcceptChar(KW) then
                Return(KW)
              else
                Return (IDENTIFIER)
            ELSE
              Return (IDENTIFIER);
          END;
        30 : 
          BEGIN
            Input2Output (yyText);
            Return (DOTDOT);
          END;
        31 : 
          BEGIN
            Input2Output (yyText);
            Return (_ASSIGNMENT_);
          END;
        32 : 
          BEGIN
            Input2Output (yyText);
            Return (LE);
          END;
        33 : 
          BEGIN
            Input2Output (yyText);
            Return (NOTEQUAL);
          END;
        34 : 
          BEGIN
            Input2Output (yyText);
            Return (GE);
          END;
        35 : 
          BEGIN
            Input2Output (yyText);
            yyText := '[';
            Return (LBRAC);
          END;
        36 : 
          BEGIN {State: Normal}
            Input2Output (yyText);
            yyText := ']';
            Return (RBRAC);
          END; 
        37 : 
          BEGIN {State: Normal}
            Input2Output (yyText);
            Return (0);
          END;
        38 : 
          BEGIN {State: Normal}
            IF yyText = #10 THEN
              Input2Output ( NEWLINE )
            ELSE
              Input2Output (yyText);
          END;

        39 : 
          Return(PLUS);
        40 : 
          Return(MINUS);
        41 : 
          Return(STAR);
        42 : 
          Return(SLASH);
        43 : 
          Return(EQUAL);
        44 : 
          Return(LT);
        45 : 
          Return(GT);
        46 : 
          Return(COMMA);
        47 : 
          Return(LPAREN);
        48 : 
          Return(RPAREN);
        49 : 
          Return(COLON);
        50 : 
          Return(SEMICOLON);
        51 : 
          Return(APE_SYMBOL);
        52 : 
          Return(UPARROW);
        53 : 
          Return(LBRAC);
        54 : 
          Return(RBRAC);
        55 : 
          Return(DOT);
        56 : 
          Return(ET);

        57 : 
          BEGIN {State: Normal}
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']);
            Return (ILLEGAL);
          END;

        58 : 
          BEGIN
            EvalCommentStart(OverreadState);
          END;
        59 : 
          BEGIN
            Return2OldState; {Could be Normal or AsmState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          END;
        60 : 
          BEGIN
            s := '';
            if FormerState = Normal then
              SwitchState (OverreadState, String2)
            else if FormerState = asmState then
              SwitchState (OverreadState,asmString1)
            else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          END;
        61 : 
          BEGIN
            if FormerState = Normal then
            else if FormerState = asmState then begin
              SwitchState (OverreadState,asmString2); s := '';
            end else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          END;
        62 : 
          BEGIN
            Input2Output (yyText);
          END;


        63 : 
          BEGIN
            IF ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) THEN BEGIN
              Input2Output (yyText);
              Return2OldState; {Could be Normal or AsmState}
            END;
          END;
        64 : 
          BEGIN
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            CommentEof;
          END;
        65 : 
          BEGIN
            Input2Output (yyText);
          END;


        66 : 
          BEGIN
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          END;
        67 : 
          BEGIN
            IF ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) THEN BEGIN
              Input2Output (yyText);
              {EvalCompilerSwitch( CompSwitch );}
              Return2OldState; {Could be Normal or AsmState or OverreadState}
            END;
          END;
        68 : 
          BEGIN
            Input2Output (yyText);
            CompSwitch := CompSwitch + yyText ;
          END;

        69 : 
          BEGIN
            Input2Output (yyText);
            s := yyText;
            Return (IMMCHAR_ID);
            Return2OldState;           
          END;
    end;
end;

//-----------------------------------------------------------------------------

// DFA table

type TTransition = record
       CharClass : set of Char;
       NextState : Integer;
     end;

const
  MarkPositionCount = 174;
  MatchCount        = 174;
  TransitionCount   = 251;
  StateCount        = 141;

  MarkPositionTable : array[0..MarkPositionCount - 1] of Integer = (
    11, 11, 0, 57, 1, 57, 47, 57, 15, 57, 28, 57, 57, 29, 57, 55, 57, 49, 57, 44, 
    57, 45, 57, 37, 38, 39, 57, 40, 57, 41, 57, 42, 57, 43, 57, 46, 57, 48, 57, 
    50, 57, 51, 57, 52, 57, 53, 57, 54, 57, 38, 57, 57, 2, 4, 4, 4, 6, 14, 13, 14, 
    9, 14, 12, 14, 7, 26, 10, 26, 26, 47, 16, 26, 17, 26, 18, 26, 51, 25, 26, 25, 
    26, 25, 26, 22, 23, 26, 50, 24, 26, 26, 26, 26, 39, 26, 40, 26, 41, 26, 42, 
    26, 43, 26, 44, 26, 45, 26, 46, 26, 48, 26, 49, 26, 52, 26, 53, 26, 54, 26, 
    55, 26, 56, 62, 58, 62, 59, 60, 62, 61, 62, 62, 66, 68, 67, 68, 68, 65, 63, 
    65, 64, 65, 69, 15, 35, 28, 27, 28, 29, 30, 36, 31, 32, 33, 34, 3, 5, 8, 16, 
    17, 18, 18, 25, 19, 25, 20, 25, 25, 58, 67, 63, 27, 27, 3, 21, 27);

  MatchTable : array [0..MatchCount - 1] of Integer = (
    11, 11, 0, 57, 1, 57, 47, 57, 15, 57, 28, 57, 57, 29, 57, 55, 57, 49, 57, 44, 
    57, 45, 57, 37, 38, 39, 57, 40, 57, 41, 57, 42, 57, 43, 57, 46, 57, 48, 57, 
    50, 57, 51, 57, 52, 57, 53, 57, 54, 57, 38, 57, 57, 2, 4, 4, 4, 6, 14, 13, 14, 
    9, 14, 12, 14, 7, 26, 10, 26, 26, 47, 16, 26, 17, 26, 18, 26, 51, 25, 26, 25, 
    26, 25, 26, 22, 23, 26, 50, 24, 26, 26, 26, 26, 39, 26, 40, 26, 41, 26, 42, 
    26, 43, 26, 44, 26, 45, 26, 46, 26, 48, 26, 49, 26, 52, 26, 53, 26, 54, 26, 
    55, 26, 56, 62, 58, 62, 59, 60, 62, 61, 62, 62, 66, 68, 67, 68, 68, 65, 63, 
    65, 64, 65, 69, 15, 35, 28, 28, 29, 30, 36, 31, 32, 33, 34, 3, 5, 8, 16, 17, 
    18, 18, 25, 19, 25, 20, 25, 25, 58, 67, 63, 27, 27, 27, 3, 21, 27);

  TransitionTable : array [0..TransitionCount - 1] of TTransition = (
    (CharClass: [#0]; NextState: 33),
    (CharClass: [#1..#9, #11..' ']; NextState: 47),
    (CharClass: [#10]; NextState: 34),
    (CharClass: ['!'..'"', '%'..'&', '?', '\', '`', '|'..#255]; NextState: 48),
    (CharClass: ['#']; NextState: 23),
    (CharClass: ['$']; NextState: 27),
    (CharClass: ['''']; NextState: 22),
    (CharClass: ['(']; NextState: 24),
    (CharClass: [')']; NextState: 41),
    (CharClass: ['*']; NextState: 37),
    (CharClass: ['+']; NextState: 35),
    (CharClass: [',']; NextState: 40),
    (CharClass: ['-']; NextState: 36),
    (CharClass: ['.']; NextState: 29),
    (CharClass: ['/']; NextState: 38),
    (CharClass: ['0'..'9']; NextState: 26),
    (CharClass: [':']; NextState: 30),
    (CharClass: [';']; NextState: 42),
    (CharClass: ['<']; NextState: 31),
    (CharClass: ['=']; NextState: 39),
    (CharClass: ['>']; NextState: 32),
    (CharClass: ['@']; NextState: 43),
    (CharClass: ['A'..'Z', '_', 'a'..'z']; NextState: 28),
    (CharClass: ['[']; NextState: 45),
    (CharClass: [']']; NextState: 46),
    (CharClass: ['^']; NextState: 44),
    (CharClass: ['{']; NextState: 25),
    (CharClass: [#0]; NextState: 33),
    (CharClass: [#1..#9, #11..' ']; NextState: 47),
    (CharClass: [#10]; NextState: 34),
    (CharClass: ['!'..'"', '%'..'&', '?', '\', '`', '|'..#255]; NextState: 48),
    (CharClass: ['#']; NextState: 23),
    (CharClass: ['$']; NextState: 27),
    (CharClass: ['''']; NextState: 22),
    (CharClass: ['(']; NextState: 24),
    (CharClass: [')']; NextState: 41),
    (CharClass: ['*']; NextState: 37),
    (CharClass: ['+']; NextState: 35),
    (CharClass: [',']; NextState: 40),
    (CharClass: ['-']; NextState: 36),
    (CharClass: ['.']; NextState: 29),
    (CharClass: ['/']; NextState: 38),
    (CharClass: ['0'..'9']; NextState: 26),
    (CharClass: [':']; NextState: 30),
    (CharClass: [';']; NextState: 42),
    (CharClass: ['<']; NextState: 31),
    (CharClass: ['=']; NextState: 39),
    (CharClass: ['>']; NextState: 32),
    (CharClass: ['@']; NextState: 43),
    (CharClass: ['A'..'Z', '_', 'a'..'z']; NextState: 28),
    (CharClass: ['[']; NextState: 45),
    (CharClass: [']']; NextState: 46),
    (CharClass: ['^']; NextState: 44),
    (CharClass: ['{']; NextState: 25),
    (CharClass: [#1..#9, #11..'"', '$'..'&', '('..#255]; NextState: 51),
    (CharClass: ['#']; NextState: 50),
    (CharClass: ['''']; NextState: 49),
    (CharClass: [#1..#9, #11..'"', '$'..'&', '('..#255]; NextState: 51),
    (CharClass: ['#']; NextState: 50),
    (CharClass: ['''']; NextState: 49),
    (CharClass: [#1..#9, #11..'&', '('..#255]; NextState: 54),
    (CharClass: [#10]; NextState: 53),
    (CharClass: ['''']; NextState: 52),
    (CharClass: [#1..#9, #11..'&', '('..#255]; NextState: 54),
    (CharClass: [#10]; NextState: 53),
    (CharClass: ['''']; NextState: 52),
    (CharClass: [#1..#9, #11..'&', '('..#255]; NextState: 54),
    (CharClass: [#10]; NextState: 53),
    (CharClass: ['''']; NextState: 55),
    (CharClass: [#1..#9, #11..'&', '('..#255]; NextState: 54),
    (CharClass: [#10]; NextState: 53),
    (CharClass: ['''']; NextState: 55),
    (CharClass: [#1..#9, #11..'!', '#'..#255]; NextState: 54),
    (CharClass: [#10]; NextState: 53),
    (CharClass: ['"']; NextState: 56),
    (CharClass: [#1..#9, #11..'!', '#'..#255]; NextState: 54),
    (CharClass: [#10]; NextState: 53),
    (CharClass: ['"']; NextState: 56),
    (CharClass: [#1..#9, #11..' ']; NextState: 68),
    (CharClass: [#10]; NextState: 66),
    (CharClass: ['!', '#', '%', '?', '\', '`', '|'..#255]; NextState: 70),
    (CharClass: ['"']; NextState: 58),
    (CharClass: ['$']; NextState: 69),
    (CharClass: ['&']; NextState: 85),
    (CharClass: ['''']; NextState: 57),
    (CharClass: ['(']; NextState: 59),
    (CharClass: [')']; NextState: 79),
    (CharClass: ['*']; NextState: 73),
    (CharClass: ['+']; NextState: 71),
    (CharClass: [',']; NextState: 78),
    (CharClass: ['-']; NextState: 72),
    (CharClass: ['.']; NextState: 84),
    (CharClass: ['/']; NextState: 74),
    (CharClass: ['0'..'1']; NextState: 63),
    (CharClass: ['2'..'7']; NextState: 64),
    (CharClass: ['8'..'9']; NextState: 65),
    (CharClass: [':']; NextState: 80),
    (CharClass: [';']; NextState: 67),
    (CharClass: ['<']; NextState: 76),
    (CharClass: ['=']; NextState: 75),
    (CharClass: ['>']; NextState: 77),
    (CharClass: ['@']; NextState: 62),
    (CharClass: ['A'..'Z', '_', 'a'..'z']; NextState: 61),
    (CharClass: ['[']; NextState: 82),
    (CharClass: [']']; NextState: 83),
    (CharClass: ['^']; NextState: 81),
    (CharClass: ['{']; NextState: 60),
    (CharClass: [#1..#9, #11..' ']; NextState: 68),
    (CharClass: [#10]; NextState: 66),
    (CharClass: ['!', '#', '%', '?', '\', '`', '|'..#255]; NextState: 70),
    (CharClass: ['"']; NextState: 58),
    (CharClass: ['$']; NextState: 69),
    (CharClass: ['&']; NextState: 85),
    (CharClass: ['''']; NextState: 57),
    (CharClass: ['(']; NextState: 59),
    (CharClass: [')']; NextState: 79),
    (CharClass: ['*']; NextState: 73),
    (CharClass: ['+']; NextState: 71),
    (CharClass: [',']; NextState: 78),
    (CharClass: ['-']; NextState: 72),
    (CharClass: ['.']; NextState: 84),
    (CharClass: ['/']; NextState: 74),
    (CharClass: ['0'..'1']; NextState: 63),
    (CharClass: ['2'..'7']; NextState: 64),
    (CharClass: ['8'..'9']; NextState: 65),
    (CharClass: [':']; NextState: 80),
    (CharClass: [';']; NextState: 67),
    (CharClass: ['<']; NextState: 76),
    (CharClass: ['=']; NextState: 75),
    (CharClass: ['>']; NextState: 77),
    (CharClass: ['@']; NextState: 62),
    (CharClass: ['A'..'Z', '_', 'a'..'z']; NextState: 61),
    (CharClass: ['[']; NextState: 82),
    (CharClass: [']']; NextState: 83),
    (CharClass: ['^']; NextState: 81),
    (CharClass: ['{']; NextState: 60),
    (CharClass: [#0]; NextState: 88),
    (CharClass: [#1..#9, #11..'!', '#'..'&', ')'..'z', '|'..#255]; NextState: 91),
    (CharClass: ['"']; NextState: 90),
    (CharClass: ['''']; NextState: 89),
    (CharClass: ['(']; NextState: 86),
    (CharClass: ['{']; NextState: 87),
    (CharClass: [#0]; NextState: 88),
    (CharClass: [#1..#9, #11..'!', '#'..'&', ')'..'z', '|'..#255]; NextState: 91),
    (CharClass: ['"']; NextState: 90),
    (CharClass: ['''']; NextState: 89),
    (CharClass: ['(']; NextState: 86),
    (CharClass: ['{']; NextState: 87),
    (CharClass: [#0]; NextState: 92),
    (CharClass: [#1..#9, #11..')', '+'..'|', '~'..#255]; NextState: 95),
    (CharClass: ['*']; NextState: 93),
    (CharClass: ['}']; NextState: 94),
    (CharClass: [#0]; NextState: 92),
    (CharClass: [#1..#9, #11..')', '+'..'|', '~'..#255]; NextState: 95),
    (CharClass: ['*']; NextState: 93),
    (CharClass: ['}']; NextState: 94),
    (CharClass: [#0]; NextState: 98),
    (CharClass: [#1..#9, #11..')', '+'..'|', '~'..#255]; NextState: 99),
    (CharClass: ['*']; NextState: 96),
    (CharClass: ['}']; NextState: 97),
    (CharClass: [#0]; NextState: 98),
    (CharClass: [#1..#9, #11..')', '+'..'|', '~'..#255]; NextState: 99),
    (CharClass: ['*']; NextState: 96),
    (CharClass: ['}']; NextState: 97),
    (CharClass: [#1..#9, #11..#255]; NextState: 100),
    (CharClass: [#1..#9, #11..#255]; NextState: 100),
    (CharClass: ['*']; NextState: 101),
    (CharClass: ['.']; NextState: 102),
    (CharClass: ['.']; NextState: 104),
    (CharClass: ['0'..'9']; NextState: 103),
    (CharClass: ['E', 'e']; NextState: 105),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 106),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 107),
    (CharClass: [')']; NextState: 109),
    (CharClass: ['.']; NextState: 108),
    (CharClass: ['=']; NextState: 110),
    (CharClass: ['=']; NextState: 111),
    (CharClass: ['>']; NextState: 112),
    (CharClass: ['=']; NextState: 113),
    (CharClass: ['$']; NextState: 115),
    (CharClass: ['0'..'9']; NextState: 114),
    (CharClass: ['''']; NextState: 116),
    (CharClass: ['''']; NextState: 117),
    (CharClass: ['*']; NextState: 118),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 119),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 121),
    (CharClass: ['@']; NextState: 120),
    (CharClass: ['0'..'1']; NextState: 122),
    (CharClass: ['2'..'7']; NextState: 124),
    (CharClass: ['8'..'9']; NextState: 127),
    (CharClass: ['A', 'C'..'F', 'a', 'c'..'f']; NextState: 126),
    (CharClass: ['B', 'b']; NextState: 123),
    (CharClass: ['O', 'o']; NextState: 125),
    (CharClass: ['0'..'7']; NextState: 124),
    (CharClass: ['8'..'9']; NextState: 127),
    (CharClass: ['A'..'F', 'a'..'f']; NextState: 126),
    (CharClass: ['O', 'o']; NextState: 125),
    (CharClass: ['0'..'9']; NextState: 127),
    (CharClass: ['A'..'F', 'a'..'f']; NextState: 126),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 128),
    (CharClass: ['*']; NextState: 129),
    (CharClass: [')']; NextState: 130),
    (CharClass: [')']; NextState: 131),
    (CharClass: ['.']; NextState: 104),
    (CharClass: ['0'..'9']; NextState: 103),
    (CharClass: ['E', 'e']; NextState: 105),
    (CharClass: [#1..'-', '/', ':'..'D', 'F'..'d', 'f'..#255]; NextState: 134),
    (CharClass: ['0'..'9']; NextState: 132),
    (CharClass: ['E', 'e']; NextState: 133),
    (CharClass: ['+', '-']; NextState: 135),
    (CharClass: ['0'..'9']; NextState: 136),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 106),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 107),
    (CharClass: ['0'..'9']; NextState: 114),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 137),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 119),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 121),
    (CharClass: ['@']; NextState: 120),
    (CharClass: ['0'..'9', '@'..'Z', '_', 'a'..'z']; NextState: 121),
    (CharClass: ['0'..'1']; NextState: 122),
    (CharClass: ['2'..'7']; NextState: 124),
    (CharClass: ['8'..'9']; NextState: 127),
    (CharClass: ['A', 'C'..'F', 'a', 'c'..'f']; NextState: 126),
    (CharClass: ['B', 'b']; NextState: 123),
    (CharClass: ['H', 'h']; NextState: 138),
    (CharClass: ['O', 'o']; NextState: 125),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 126),
    (CharClass: ['H', 'h']; NextState: 138),
    (CharClass: ['0'..'7']; NextState: 124),
    (CharClass: ['8'..'9']; NextState: 127),
    (CharClass: ['A'..'F', 'a'..'f']; NextState: 126),
    (CharClass: ['H', 'h']; NextState: 138),
    (CharClass: ['O', 'o']; NextState: 125),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 126),
    (CharClass: ['H', 'h']; NextState: 138),
    (CharClass: ['0'..'9']; NextState: 127),
    (CharClass: ['A'..'F', 'a'..'f']; NextState: 126),
    (CharClass: ['H', 'h']; NextState: 138),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 128),
    (CharClass: [#1..'-', '/', ':'..'D', 'F'..'d', 'f'..#255]; NextState: 134),
    (CharClass: ['0'..'9']; NextState: 132),
    (CharClass: ['E', 'e']; NextState: 133),
    (CharClass: ['+', '-']; NextState: 139),
    (CharClass: ['0'..'9']; NextState: 136),
    (CharClass: ['0'..'9']; NextState: 136),
    (CharClass: [#1..'-', '/', ':'..#255]; NextState: 134),
    (CharClass: ['0'..'9']; NextState: 140),
    (CharClass: ['0'..'9', 'A'..'F', 'a'..'f']; NextState: 137),
    (CharClass: ['0'..'9']; NextState: 136),
    (CharClass: [#1..'-', '/', ':'..#255]; NextState: 134),
    (CharClass: ['0'..'9']; NextState: 140)
    );

  MarksLow : array [0..StateCount-1] of Integer = (
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 4, 6, 8, 
    10, 12, 13, 15, 17, 19, 21, 23, 24, 25, 27, 29, 31, 33, 35, 37, 39, 41, 43, 
    45, 47, 49, 51, 52, 54, 55, 56, 58, 59, 60, 62, 64, 66, 68, 70, 72, 74, 77, 
    79, 81, 83, 84, 87, 89, 90, 91, 93, 95, 97, 99, 101, 103, 105, 107, 109, 111, 
    113, 115, 117, 119, 121, 122, 124, 125, 127, 129, 130, 131, 132, 134, 135, 136, 
    138, 139, 140, 141, 142, 143, 144, 145, 145, 146, 147, 148, 149, 150, 151, 152, 
    153, 154, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 164, 165, 166, 
    167, 168, 169, 170, 170, 170, 170, 171, 172, 173, 173);

  MarksHigh : array [0..StateCount-1] of Integer = (
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
    3, 5, 7, 9, 11, 12, 14, 16, 18, 20, 22, 23, 24, 26, 28, 30, 32, 34, 36, 38, 
    40, 42, 44, 46, 48, 50, 51, 53, 54, 55, 57, 58, 59, 61, 63, 65, 67, 69, 71, 
    73, 76, 78, 80, 82, 83, 86, 88, 89, 90, 92, 94, 96, 98, 100, 102, 104, 106, 
    108, 110, 112, 114, 116, 118, 120, 121, 123, 124, 126, 128, 129, 130, 131, 133, 
    134, 135, 137, 138, 139, 140, 141, 142, 143, 144, 144, 145, 146, 147, 148, 149, 
    150, 151, 152, 153, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 163, 
    164, 165, 166, 167, 168, 169, 169, 169, 169, 170, 171, 172, 172, 173);

  MatchesLow : array [0..StateCount-1] of Integer = (
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 4, 6, 8, 
    10, 12, 13, 15, 17, 19, 21, 23, 24, 25, 27, 29, 31, 33, 35, 37, 39, 41, 43, 
    45, 47, 49, 51, 52, 54, 55, 56, 58, 59, 60, 62, 64, 66, 68, 70, 72, 74, 77, 
    79, 81, 83, 84, 87, 89, 90, 91, 93, 95, 97, 99, 101, 103, 105, 107, 109, 111, 
    113, 115, 117, 119, 121, 122, 124, 125, 127, 129, 130, 131, 132, 134, 135, 136, 
    138, 139, 140, 141, 142, 143, 144, 144, 144, 145, 146, 147, 148, 149, 150, 151, 
    152, 153, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 163, 164, 165, 
    166, 167, 168, 169, 170, 171, 171, 171, 172, 173, 173);

  MatchesHigh : array [0..StateCount-1] of Integer = (
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
    3, 5, 7, 9, 11, 12, 14, 16, 18, 20, 22, 23, 24, 26, 28, 30, 32, 34, 36, 38, 
    40, 42, 44, 46, 48, 50, 51, 53, 54, 55, 57, 58, 59, 61, 63, 65, 67, 69, 71, 
    73, 76, 78, 80, 82, 83, 86, 88, 89, 90, 92, 94, 96, 98, 100, 102, 104, 106, 
    108, 110, 112, 114, 116, 118, 120, 121, 123, 124, 126, 128, 129, 130, 131, 133, 
    134, 135, 137, 138, 139, 140, 141, 142, 143, 143, 143, 144, 145, 146, 147, 148, 
    149, 150, 151, 152, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 162, 
    163, 164, 165, 166, 167, 168, 169, 170, 170, 170, 171, 172, 172, 173);

  TransitionsLow : array [0..StateCount-1] of Integer = (
    0, 0, 0, 27, 54, 57, 60, 63, 66, 69, 72, 75, 78, 107, 136, 142, 148, 152, 156, 
    160, 164, 165, 166, 166, 166, 168, 168, 171, 172, 173, 175, 176, 178, 179, 179, 
    179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 179, 
    181, 181, 182, 182, 182, 183, 183, 183, 183, 184, 184, 185, 187, 193, 197, 199, 
    199, 199, 199, 200, 200, 200, 200, 200, 200, 200, 200, 200, 200, 200, 200, 200, 
    200, 200, 200, 200, 201, 201, 201, 201, 201, 201, 201, 202, 202, 202, 203, 203, 
    203, 203, 203, 203, 203, 206, 209, 211, 212, 213, 213, 213, 213, 213, 213, 213, 
    214, 215, 215, 215, 215, 216, 218, 219, 226, 228, 233, 233, 235, 238, 239, 239, 
    239, 239, 242, 244, 244, 245, 247, 248, 248, 249);

  TransitionsHigh : array [0..StateCount-1] of Integer = (
    -1, -1, 26, 53, 56, 59, 62, 65, 68, 71, 74, 77, 106, 135, 141, 147, 151, 155, 
    159, 163, 164, 165, 165, 165, 167, 167, 170, 171, 172, 174, 175, 177, 178, 178, 
    178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 178, 
    180, 180, 181, 181, 181, 182, 182, 182, 182, 183, 183, 184, 186, 192, 196, 198, 
    198, 198, 198, 199, 199, 199, 199, 199, 199, 199, 199, 199, 199, 199, 199, 199, 
    199, 199, 199, 199, 200, 200, 200, 200, 200, 200, 200, 201, 201, 201, 202, 202, 
    202, 202, 202, 202, 202, 205, 208, 210, 211, 212, 212, 212, 212, 212, 212, 212, 
    213, 214, 214, 214, 214, 215, 217, 218, 225, 227, 232, 232, 234, 237, 238, 238, 
    238, 238, 241, 243, 243, 244, 246, 247, 247, 248, 250);

var yyn : Integer;

label start, scan, action;

begin

start:
  // initialize
  yynew;
scan:
  // mark positions and matches
  for yyn := yykl[yystate] to     yykh[yystate] do yymark(yyk[yyn]);
  for yyn := yymh[yystate] downto yyml[yystate] do yymatch(yym[yyn]);

  if yytl[yystate] > yyth[yystate] then goto action;  dead state

  // get next character
  yyscan;

  // determine action
  yyn := yytl[yystate];
  while (yyn<=yyth[yystate]) and not (yyactchar in yyt[yyn].cc) do Inc(yyn);
  if yyn > yyth[yystate] then goto action;
    // no transition on yyactchar in this state

  // switch to new state:
  yystate := yyt[yyn].s;
  goto scan;

action:
  // execute action
  if yyfind(yyrule) then
  begin
    yyaction(yyrule);
    if yyreject then goto action;
  end
  else
    if not yydefault and yywrap then
    begin
      yyclear;
      return(0);
    end;

  if not yydone then goto start;

  yylex := yyretval;
end;

//-----------------------------------------------------------------------------

