unit WindowList;

// 13-NOV-99 ml: structural rework
// 13-NOV-99 ml: implementation so far
//----------------------------------------------------------------------------------------------------------------------
// This unit contains a simple form used to give the user quick access to all opened
// project windows or to allow to reshow a project which is loaded but whose window has been
// closed.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, 
  StdCtrls, DCGData;

type
  TWindowListForm = class(TForm)
    WindowListBox: TListBox;
    procedure WindowListBoxDblClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure WindowListBoxKeyPress(Sender: TObject; var Key: Char);
  public
    procedure ActivateWindow(const AForm: TBaseProjectForm); overload;
    procedure ActivateWindow(const AProject: String); overload;
    procedure ActivateWindow; overload;
    procedure AddWindow(const AForm: TBaseProjectForm);
    function ProjectIsOpen(const Name: String; var Project: TBaseProjectForm): Boolean;
    procedure RemoveWindow(const AForm: TBaseProjectForm);
    procedure UpdateList;
  end;

var
  WindowListForm: TWindowListForm;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  Main, Project, DCGTools;

{$R *.DFM}

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.AddWindow(const AForm: TBaseProjectForm);

begin
  if WindowListBox.Items.IndexOfObject(AForm) = -1 then WindowListBox.Items.AddObject(PureName(AForm.WorkingFile), AForm);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.RemoveWindow(const AForm: TBaseProjectForm);

var
  Index: Integer;

begin
  Index := WindowListBox.Items.IndexOfObject(AForm);
  if Index > -1 then WindowListBox.Items.Delete(Index);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.WindowListBoxDblClick(Sender: TObject);

begin
  ActivateWindow;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.UpdateList;

var
  Index: Integer;

begin
  for Index := 0 to WindowListBox.Items.Count-1 do
    WindowListBox.Items[Index] := PureName((WindowListBox.Items.Objects[Index] as TBaseProjectForm).WorkingFile);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.FormClose(Sender: TObject; var Action: TCloseAction);

begin
  MainForm.WindowListAction.checked := False;
end;

//----------------------------------------------------------------------------------------------------------------------

function TWindowListForm.ProjectIsOpen(const Name: String; var Project: TBaseProjectForm): Boolean;

var
  I: Integer;

begin
  Result := False;
  Project := nil;
  for I := 0 to WindowListBox.Items.Count-1 do
    if CompareText((WindowListBox.Items.Objects[I] as TBaseProjectForm).WorkingFile, Name) = 0 then
    begin
      Project := WindowListBox.Items.Objects[I] as TBaseProjectForm;
      Result := True;
      Break;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.ActivateWindow(const AForm: TBaseProjectForm);

var
  Index: Integer;

begin
  Index := WindowListBox.Items.IndexOfObject(AForm);
  WindowListBox.ItemIndex := Index;
  ActivateWindow;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.ActivateWindow(const AProject: String);

var
  I, Index : Integer;

begin
  Index := -1;
  for I := 0 to WindowListBox.Items.Count - 1 do
    if CompareText((WindowListBox.Items.Objects[I] as TBaseProjectForm).WorkingFile, AProject) = 0 then
    begin
      Index := I;
      Break;
    end;

  WindowListBox.ItemIndex := Index;
  ActivateWindow;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.ActivateWindow;

begin
  if WindowListBox.ItemIndex > -1 then
    with (WindowListBox.Items.Objects[WindowListBox.ItemIndex] as TBaseProjectForm) do
    begin
      if WindowState = wsMinimized then WindowState := wsNormal;
      Show;
      BringToFront;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TWindowListForm.WindowListBoxKeyPress(Sender: TObject; var Key: Char);

begin
  if Key = #13 then
  begin
    Key := #0;
    ActivateWindow;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

end.

